@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -25,9 +25,8 @@
 
 /// @file
 
-#include <tr1/unordered_map>
-#include <tr1/unordered_set>
 #include <ostream>
+#include "abg-cxx-compat.h"
 #include "abg-corpus.h"
 #include "abg-diff-utils.h"
 #include "abg-reporter.h"
@@ -52,8 +51,8 @@ typedef std::vector<filter_base_sptr> filters;
 // Inject types we need into this namespace.
 using std::ostream;
 using std::vector;
-using std::tr1::unordered_map;
-using std::tr1::unordered_set;
+using abg_compat::unordered_map;
+using abg_compat::unordered_set;
 using std::pair;
 
 using diff_utils::insertion;
@@ -145,6 +144,10 @@ typedef unordered_map<size_t, size_t> pointer_map;
 /// value is a @ref decl_base_sptr.
 typedef unordered_map<string, decl_base_sptr> string_decl_base_sptr_map;
 
+/// Convenience typedef for a map which key is a string and which
+/// value is a @ref type_base_sptr.
+typedef unordered_map<string, type_base_sptr> string_type_base_sptr_map;
+
 /// Convenience typedef for a map which key is an unsigned integer and
 /// which value is a @ref decl_base_sptr
 typedef unordered_map<unsigned, decl_base_sptr> unsigned_decl_base_sptr_map;
@@ -372,59 +375,61 @@ enum diff_category
   /// alias change that is harmless.
   HARMLESS_SYMBOL_ALIAS_CHANGE_CATEORY = 1 << 6,
 
+  HARMLESS_UNION_CHANGE_CATEGORY = 1 << 7,
+
   /// This means that a diff node was marked as suppressed by a
   /// user-provided suppression specification.
-  SUPPRESSED_CATEGORY = 1 << 7,
+  SUPPRESSED_CATEGORY = 1 << 8,
 
   /// This means that a diff node was warked as being for a private
   /// type.  That is, the diff node is meant to be suppressed by a
   /// suppression specification that was auto-generated to filter out
   /// changes to private types.
-  PRIVATE_TYPE_CATEGORY = 1 << 8,
+  PRIVATE_TYPE_CATEGORY = 1 << 9,
 
   /// This means the diff node (or at least one of its descendant
   /// nodes) carries a change that modifies the size of a type or an
   /// offset of a type member.  Removal or changes of enumerators in a
   /// enum fall in this category too.
-  SIZE_OR_OFFSET_CHANGE_CATEGORY = 1 << 9,
+  SIZE_OR_OFFSET_CHANGE_CATEGORY = 1 << 10,
 
   /// This means that a diff node in the sub-tree carries an
   /// incompatible change to a vtable.
-  VIRTUAL_MEMBER_CHANGE_CATEGORY = 1 << 10,
+  VIRTUAL_MEMBER_CHANGE_CATEGORY = 1 << 11,
 
   /// A diff node in this category is redundant.  That means it's
   /// present as a child of a other nodes in the diff tree.
-  REDUNDANT_CATEGORY = 1 << 11,
+  REDUNDANT_CATEGORY = 1 << 12,
 
   /// This means that a diff node in the sub-tree carries a class type
   /// that was declaration-only and that is now defined, or vice
   /// versa.
-  CLASS_DECL_ONLY_DEF_CHANGE_CATEGORY = 1 << 12,
+  CLASS_DECL_ONLY_DEF_CHANGE_CATEGORY = 1 << 13,
 
   /// A diff node in this category is a function parameter type which
   /// top cv-qualifiers change.
-  FN_PARM_TYPE_TOP_CV_CHANGE_CATEGORY = 1 << 13,
+  FN_PARM_TYPE_TOP_CV_CHANGE_CATEGORY = 1 << 14,
 
   /// A diff node in this category has a function parameter type with a
   /// cv-qualifiers change.
-  FN_PARM_TYPE_CV_CHANGE_CATEGORY = 1 << 14,
+  FN_PARM_TYPE_CV_CHANGE_CATEGORY = 1 << 15,
 
   /// A diff node in this category is a function return type with a
   /// cv-qualifier change.
-  FN_RETURN_TYPE_CV_CHANGE_CATEGORY = 1 << 15,
+  FN_RETURN_TYPE_CV_CHANGE_CATEGORY = 1 << 16,
 
   /// A diff node in this category is for a variable which type holds
   /// a cv-qualifier change.
-  VAR_TYPE_CV_CHANGE_CATEGORY = 1 << 16,
+  VAR_TYPE_CV_CHANGE_CATEGORY = 1 << 17,
 
   /// A diff node in this category carries a change from void pointer
   /// to non-void pointer.
-  VOID_PTR_TO_PTR_CHANGE_CATEGORY = 1 << 17,
+  VOID_PTR_TO_PTR_CHANGE_CATEGORY = 1 << 18,
 
   /// A diff node in this category carries a change in the size of the
   /// array type of a global variable, but the ELF size of the
   /// variable didn't change.
-  BENIGN_INFINITE_ARRAY_CHANGE_CATEGORY = 1 << 18,
+  BENIGN_INFINITE_ARRAY_CHANGE_CATEGORY = 1 << 19,
   /// A special enumerator that is the logical 'or' all the
   /// enumerators above.
   ///
@@ -438,6 +443,7 @@ enum diff_category
   | STATIC_DATA_MEMBER_CHANGE_CATEGORY
   | HARMLESS_ENUM_CHANGE_CATEGORY
   | HARMLESS_SYMBOL_ALIAS_CHANGE_CATEORY
+  | HARMLESS_UNION_CHANGE_CATEGORY
   | SUPPRESSED_CATEGORY
   | PRIVATE_TYPE_CATEGORY
   | SIZE_OR_OFFSET_CHANGE_CATEGORY
@@ -828,6 +834,10 @@ public:
   void
   show_added_symbols_unreferenced_by_debug_info(bool f);
 
+  void show_unreachable_types(bool f);
+
+  bool show_unreachable_types();
+
   bool
   show_impacted_interfaces() const;
 
@@ -2278,6 +2288,8 @@ public:
 
   class diff_stats;
 
+  virtual ~corpus_diff() {}
+
   /// A convenience typedef for a shared pointer to @ref diff_stats
   typedef shared_ptr<diff_stats> diff_stats_sptr;
 
@@ -2341,6 +2353,24 @@ public:
   const string_elf_symbol_map&
   added_unrefed_variable_symbols() const;
 
+  const string_type_base_sptr_map&
+  deleted_unreachable_types() const;
+
+  const vector<type_base_sptr>&
+  deleted_unreachable_types_sorted() const;
+
+  const string_type_base_sptr_map&
+  added_unreachable_types() const;
+
+  const vector<type_base_sptr>&
+  added_unreachable_types_sorted() const;
+
+  const string_diff_sptr_map&
+  changed_unreachable_types() const;
+
+  const vector<diff_sptr>&
+  changed_unreachable_types_sorted() const;
+
   const diff_context_sptr
   context() const;
 
@@ -2388,6 +2418,12 @@ public:
   friend void
   apply_suppressions(const corpus_diff* diff_tree);
 
+  friend void
+  maybe_report_unreachable_type_changes(const corpus_diff& d,
+					const corpus_diff::diff_stats &s,
+					const string& indent,
+					ostream& out);
+
   friend class default_reporter;
   friend class leaf_reporter;
 }; // end class corpus_diff
@@ -2529,6 +2565,28 @@ public:
   size_t num_leaf_var_changes_filtered_out() const;
   void num_leaf_var_changes_filtered_out(size_t);
   size_t net_num_leaf_var_changes() const;
+
+  size_t num_added_unreachable_types() const;
+  void num_added_unreachable_types(size_t);
+
+  size_t num_added_unreachable_types_filtered_out() const;
+  void num_added_unreachable_types_filtered_out(size_t);
+  size_t net_num_added_unreachable_types() const;
+
+  size_t num_removed_unreachable_types() const;
+  void num_removed_unreachable_types(size_t);
+
+  size_t num_removed_unreachable_types_filtered_out() const;
+  void num_removed_unreachable_types_filtered_out(size_t);
+  size_t net_num_removed_unreachable_types() const;
+
+  size_t num_changed_unreachable_types() const;
+  void num_changed_unreachable_types(size_t);
+
+  size_t num_changed_unreachable_types_filtered_out() const;
+  void num_changed_unreachable_types_filtered_out(size_t);
+  size_t net_num_changed_unreachable_types() const;
+
 }; // end class corpus_diff::diff_stats
 
 /// The base class for the node visitors.  These are the types used to
@@ -2544,6 +2602,8 @@ public:
 
   diff_node_visitor();
 
+  virtual ~diff_node_visitor() {}
+
   diff_node_visitor(visiting_kind k);
 
   visiting_kind
@@ -2793,7 +2853,10 @@ const diff*
 peel_qualified_diff(const diff* dif);
 
 const diff*
-peel_pointer_or_qualified_type(const diff*dif);
+peel_pointer_or_qualified_type_diff(const diff* dif);
+
+const diff*
+peel_typedef_or_qualified_type_diff(const diff* dif);
 }// end namespace comparison
 
 }// end namespace abigail
