@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -64,7 +64,8 @@ public:
   {
     ARTIFICIAL_ORIGIN = 0,
     NATIVE_XML_ORIGIN,
-    DWARF_ORIGIN
+    DWARF_ORIGIN,
+    LINUX_KERNEL_BINARY_ORIGIN
   };
 
 private:
@@ -72,12 +73,15 @@ private:
 
   void record_canonical_type(const type_base_sptr&) const;
   type_base_sptr lookup_canonical_type(const string& qualified_name) const;
+  void set_group(corpus_group*);
 
 public:
   shared_ptr<priv> priv_;
 
   corpus(ir::environment*, const string& path= "");
 
+  virtual ~corpus() {}
+
   const environment*
   get_environment() const;
 
@@ -111,6 +115,24 @@ public:
   const type_maps&
   get_type_per_loc_map() const;
 
+  virtual bool
+  recording_types_reachable_from_public_interface_supported();
+
+  void
+  record_type_as_reachable_from_public_interfaces(const type_base&);
+
+  bool
+  type_is_reachable_from_public_interfaces(const type_base&) const;
+
+  const vector<type_base_wptr>&
+  get_types_not_reachable_from_public_interfaces() const;
+
+  const corpus_group*
+  get_group() const;
+
+  corpus_group*
+  get_group();
+
   origin
   get_origin() const;
 
@@ -162,7 +184,7 @@ public:
   const string_elf_symbols_map_sptr
   get_fun_symbol_map_sptr() const;
 
-  const string_elf_symbols_map_type&
+  virtual const string_elf_symbols_map_type&
   get_fun_symbol_map() const;
 
   const string_elf_symbols_map_sptr
@@ -180,7 +202,7 @@ public:
   const string_elf_symbols_map_sptr
   get_var_symbol_map_sptr() const;
 
-  const string_elf_symbols_map_type&
+  virtual const string_elf_symbols_map_type&
   get_var_symbol_map() const;
 
   const string_elf_symbols_map_sptr
@@ -279,6 +301,7 @@ public:
   get_exported_decls_builder() const;
 
   friend class type_base;
+  friend class corpus_group;
 };// end class corpus.
 
 /// Abstracts the building of the set of exported variables and
@@ -362,6 +385,12 @@ public:
   const corpora_type&
   get_corpora() const;
 
+  const corpus_sptr
+  get_main_corpus() const;
+
+  corpus_sptr
+  get_main_corpus();
+
   virtual bool
   is_empty() const;
 
@@ -371,10 +400,10 @@ public:
   virtual const corpus::variables&
   get_variables() const;
 
-  const string_elf_symbols_map_type&
+  virtual const string_elf_symbols_map_type&
   get_var_symbol_map() const;
 
-  const string_elf_symbols_map_type&
+  virtual const string_elf_symbols_map_type&
   get_fun_symbol_map() const;
 
   virtual const elf_symbols&
@@ -389,6 +418,12 @@ public:
   virtual const elf_symbols&
   get_unreferenced_variable_symbols() const;
 
+  unordered_set<interned_string, hash_interned_string>*
+  get_public_types_pretty_representations();
+
+  virtual bool
+  recording_types_reachable_from_public_interface_supported();
+
   bool
   operator==(const corpus_group&) const;
 }; // end class corpus_group
