@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -43,7 +43,7 @@
 #include <string>
 #include <vector>
 #include <sstream>
-#include <tr1/memory>
+#include "abg-cxx-compat.h"
 #include "abg-fwd.h"
 
 namespace abigail
@@ -57,7 +57,7 @@ namespace abigail
 namespace diff_utils
 {
 
-using std::tr1::shared_ptr;
+using abg_compat::shared_ptr;
 
 // Inject the names from std:: below into this namespace
 using std::string;
@@ -186,7 +186,7 @@ public:
   operator--(int)
   {
     point tmp(*this);
-    (*this)--;
+    --(*this);
     return tmp;
   }
 
@@ -194,7 +194,7 @@ public:
   operator++(int)
   {
     point tmp(*this);
-    (*this)++;
+    ++(*this);
     return tmp;
   }
 
@@ -772,12 +772,36 @@ struct deep_ptr_eq_functor
     return *first == *second;
   }
 
+  /// This equality operator compares pointers by comparing the
+  /// pointed-to objects.
+  ///
+  /// @param first the first comparison argument.
+  ///
+  /// @param second the second comparison argument.
+  ///
+  /// @return true if the objects pointed to by the pointers are
+  /// equal, false otherwise.
   template<typename T>
   bool
   operator()(const shared_ptr<T> first,
 	     const shared_ptr<T> second) const
   {return operator()(first.get(), second.get());}
-};
+
+  /// This equality operator compares pointers by comparing the
+  /// pointed-to objects.
+  ///
+  /// @param first the first comparison argument.
+  ///
+  /// @param second the second comparison argument.
+  ///
+  /// @return true if the objects pointed to by the pointers are
+  /// equal, false otherwise.
+  template<typename T>
+  bool
+  operator()(const weak_ptr<T> first,
+	     const weak_ptr<T> second) const
+  {return operator()(shared_ptr<T>(first), shared_ptr<T>(second));}
+}; // end struct deep_ptr_eq_functor
 
 /// Find the end of the furthest reaching d-path on diagonal k, for
 /// two sequences.  In the paper This is referred to as "the basic
