@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -26,15 +26,13 @@
 #include <stdint.h>
 #include <cstdlib>
 #include <cstddef>
-#include <tr1/memory>
 #include <list>
 #include <vector>
 #include <string>
-#include <tr1/functional>
-#include <tr1/unordered_map>
 #include <typeinfo>
 #include <utility> // for std::rel_ops, at least.
 #include <ostream>
+#include "abg-cxx-compat.h"
 #include "abg-interned-str.h"
 #include "abg-hash.h"
 
@@ -65,9 +63,9 @@ namespace abigail
 */
 
 // Inject some types.
-using std::tr1::shared_ptr;
-using std::tr1::weak_ptr;
-using std::tr1::unordered_map;
+using abg_compat::shared_ptr;
+using abg_compat::weak_ptr;
+using abg_compat::unordered_map;
 using std::string;
 using std::vector;
 
@@ -87,7 +85,7 @@ typedef shared_ptr<corpus_group> corpus_group_sptr;
 
 // Forward declarations for ir.
 
-struct ir_node_visitor;
+class  ir_node_visitor;
 
 struct ir_traversable_base;
 
@@ -102,10 +100,6 @@ typedef shared_ptr<environment> environment_sptr;
 class location;
 class location_manager;
 
-class corpus;
-/// A convenience typedef for shared pointer to @ref corpus.
-typedef shared_ptr<corpus> corpus_sptr;
-
 class type_or_decl_base;
 /// A convenience typedef for a shared_ptr to @ref type_or_decl_base.
 typedef shared_ptr<type_or_decl_base> type_or_decl_base_sptr;
@@ -392,6 +386,12 @@ is_decl(const type_or_decl_base*);
 decl_base_sptr
 is_decl(const type_or_decl_base_sptr&);
 
+decl_base*
+is_decl_slow(const type_or_decl_base*);
+
+decl_base_sptr
+is_decl_slow(const type_or_decl_base_sptr&);
+
 bool
 is_type(const type_or_decl_base&);
 
@@ -408,19 +408,13 @@ bool
 is_anonymous_type(const type_base_sptr&);
 
 const type_decl*
-is_type_decl(const type_base*);
+is_type_decl(const type_or_decl_base*);
 
 type_decl_sptr
-is_type_decl(const type_base_sptr&);
-
-type_decl_sptr
-is_type_decl(const decl_base_sptr&);
+is_type_decl(const type_or_decl_base_sptr&);
 
 typedef_decl_sptr
-is_typedef(const type_base_sptr);
-
-typedef_decl_sptr
-is_typedef(const decl_base_sptr);
+is_typedef(const type_or_decl_base_sptr);
 
 const typedef_decl*
 is_typedef(const type_base*);
@@ -431,6 +425,9 @@ is_typedef(type_base*);
 enum_type_decl_sptr
 is_enum_type(const type_or_decl_base_sptr&);
 
+const enum_type_decl*
+is_enum_type(const type_or_decl_base*);
+
 bool
 is_class_type(const type_or_decl_base&);
 
@@ -509,6 +506,9 @@ is_method_type(const type_or_decl_base*);
 method_type*
 is_method_type(type_or_decl_base*);
 
+class_or_union_sptr
+look_through_decl_only_class(const class_or_union&);
+
 class_or_union_sptr
 look_through_decl_only_class(class_or_union_sptr);
 
@@ -568,6 +568,12 @@ is_scope_decl(const decl_base_sptr&);
 bool
 is_member_type(const type_base_sptr&);
 
+bool
+is_user_defined_type(const type_base*);
+
+bool
+is_user_defined_type(const type_base_sptr&);
+
 void
 remove_decl_from_scope(decl_base_sptr);
 
@@ -655,6 +661,12 @@ is_array_type(const type_or_decl_base* decl);
 array_type_def_sptr
 is_array_type(const type_or_decl_base_sptr& decl);
 
+array_type_def_sptr
+is_array_of_qualified_element(type_base_sptr&);
+
+qualified_type_def_sptr
+is_array_of_qualified_element(const array_type_def_sptr&);
+
 void
 set_data_member_offset(var_decl_sptr, uint64_t);
 
@@ -800,6 +812,10 @@ type_base*
 peel_typedef_pointer_or_reference_type(const type_base* type,
 				       bool peel_qualified_type = true);
 
+type_base*
+peel_pointer_or_reference_type(const type_base *type,
+			       bool peel_qualified_type = true);
+
 string
 get_name(const type_or_decl_base*, bool qualified = true);
 
@@ -913,17 +929,20 @@ get_pretty_representation(const method_type_sptr&,
 string
 get_class_or_union_flat_representation(const class_or_union& cou,
 				       const string& indent,
-				       bool one_line);
+				       bool one_line,
+				       bool qualified_name = true);
 
 string
 get_class_or_union_flat_representation(const class_or_union* cou,
 				       const string& indent,
-				       bool one_line);
+				       bool one_line,
+				       bool qualified_name = true);
 
 string
 get_class_or_union_flat_representation(const class_or_union_sptr& cou,
 				       const string& indent,
-				       bool one_line);
+				       bool one_line,
+				       bool qualified_name = true);
 
 bool
 odr_is_relevant(const type_or_decl_base&);
@@ -1247,6 +1266,9 @@ type_or_void(const type_base_sptr, const environment*);
 type_base_sptr
 canonicalize(type_base_sptr);
 
+type_base_sptr
+re_canonicalize(type_base_sptr);
+
 type_base*
 type_has_non_canonicalized_subtype(type_base_sptr t);
 
@@ -1257,6 +1279,9 @@ type_has_sub_type_changes(type_base_sptr t_v1,
 void
 keep_type_alive(type_base_sptr t);
 
+size_t
+hash_type(const type_base *t);
+
 size_t
 hash_type_or_decl(const type_or_decl_base *);
 
@@ -1288,7 +1313,7 @@ typedef shared_ptr<suppression_base> suppression_sptr;
 /// Convenience typedef for a vector of @ref suppression_sptr
 typedef vector<suppression_sptr> suppressions_type;
 
-} // end namespace comparison
+} // end namespace suppr
 
 void
 dump(const decl_base_sptr, std::ostream&);
@@ -1333,7 +1358,7 @@ dump_decl_location(const decl_base_sptr&);
 /// This is a wrapper around the 'assert' glibc call.  It allows for
 /// its argument to have side effects, so that it keeps working when
 /// the code of libabigail is compiled with the NDEBUG macro defined.
-#define ABG_ASSERT(cond) do {({bool __abg_cond__ = (cond); assert(__abg_cond__); !!__abg_cond__;});} while (false)
+#define ABG_ASSERT(cond) do {({bool __abg_cond__ = bool(cond); assert(__abg_cond__); !!__abg_cond__;});} while (false)
 #endif
 
 } // end namespace abigail
