@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2016-2019 Red Hat, Inc.
+// Copyright (C) 2016-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -23,7 +23,7 @@
 #ifndef __ABG_SUPPRESSION_H__
 #define __ABG_SUPPRESSION_H__
 
-#include <tr1/unordered_set>
+#include "abg-cxx-compat.h"
 #include "abg-ini.h"
 #include "abg-comparison.h"
 
@@ -43,7 +43,7 @@ namespace suppr
 {
 
 using namespace abigail::comparison;
-using std::tr1::unordered_set;
+using abg_compat::unordered_set;
 
 /// Base type of the suppression specifications types.
 ///
@@ -97,6 +97,9 @@ public:
   const string&
   get_file_name_not_regex_str() const;
 
+  bool
+  has_file_name_related_property() const;
+
   void
   set_soname_regex_str(const string& regexp);
 
@@ -109,10 +112,22 @@ public:
   const string&
   get_soname_not_regex_str() const;
 
+  bool
+  has_soname_related_property() const;
+
   virtual bool
   suppresses_diff(const diff*) const = 0;
 
   virtual ~suppression_base();
+
+  friend bool
+  suppression_matches_soname(const string& soname,
+			     const suppression_base& suppr);
+
+  friend bool
+  suppression_matches_soname_or_filename(const string& soname,
+					 const string& filename,
+					 const suppression_base& suppr);
 }; // end class suppression_base
 
 void
@@ -200,6 +215,12 @@ public:
   const string&
   get_type_name_regex_str() const;
 
+  void
+  set_type_name_not_regex_str(const string& name_regex_str);
+
+  const string&
+  get_type_name_not_regex_str() const;
+
   void
   set_type_name(const string& name);
 
@@ -512,6 +533,12 @@ public:
   void
   set_symbol_name_regex_str(const string&);
 
+  const string&
+  get_symbol_name_not_regex_str() const;
+
+  void
+  set_symbol_name_not_regex_str(const string&);
+
   const string&
   get_symbol_version() const;
 
@@ -701,6 +728,12 @@ public:
   void
   set_symbol_name_regex_str(const string&);
 
+  const string&
+  get_symbol_name_not_regex_str() const;
+
+  void
+  set_symbol_name_not_regex_str(const string&);
+
   const string&
   get_symbol_version() const;
 
@@ -802,6 +835,15 @@ file_suppression_sptr
 file_is_suppressed(const string& file_path,
 		   const suppressions_type& suppressions);
 
+bool
+suppression_matches_soname(const string& soname,
+			   const suppression_base& suppr);
+
+bool
+suppression_matches_soname_or_filename(const string& soname,
+				       const string& filename,
+				       const suppression_base& suppr);
+
 const char*
 get_private_types_suppr_spec_label();
 
