@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -20,13 +20,16 @@
 
 ///@file
 
-#include <tr1/memory>
+#ifndef __ABG_TOOLS_UTILS_H
+#define __ABG_TOOLS_UTILS_H
+
 #include <string>
 #include <set>
 #include <ostream>
 #include <istream>
 #include <iostream>
-#include <abg-suppression.h>
+#include "abg-cxx-compat.h"
+#include "abg-suppression.h"
 
 namespace abigail
 {
@@ -39,14 +42,19 @@ using std::istream;
 using std::ifstream;
 using std::string;
 using std::set;
-using std::tr1::shared_ptr;
+using abg_compat::shared_ptr;
 
 const char* get_system_libdir();
+const char* get_anonymous_struct_internal_name_prefix();
+const char* get_anonymous_union_internal_name_prefix();
+const char* get_anonymous_enum_internal_name_prefix();
+
 bool file_exists(const string&);
 bool is_regular_file(const string&);
 bool is_dir(const string&);
 bool dir_exists(const string&);
 bool dir_is_empty(const string &);
+bool decl_names_equal(const string&, const string&);
 bool maybe_get_symlink_target_file_path(const string& file_path,
 					string& target_path);
 bool base_name(string const& path,
@@ -78,9 +86,9 @@ void convert_char_stars_to_char_star_stars(const vector<char*>&,
 suppr::type_suppression_sptr
 gen_suppr_spec_from_headers(const string& hdrs_root_dir);
 
-bool
-gen_suppr_spec_from_kernel_abi_whitelist(const string& abi_whitelist_path,
-					 suppr::suppressions_type& s);
+suppr::suppressions_type
+gen_suppr_spec_from_kernel_abi_whitelists
+   (const vector<string>& abi_whitelist_paths);
 
 bool
 get_vmlinux_path_from_kernel_dist(const string&	from,
@@ -124,7 +132,7 @@ typedef shared_ptr<temp_file> temp_file_sptr;
 /// This is a helper file around the  mkstemp API.
 ///
 /// Once the temporary file is created, users can interact with it
-/// using an iostream.  They can also get the path to the newly
+/// using an fstream.  They can also get the path to the newly
 /// created temporary file.
 ///
 /// When the instance of @ref temp_file is destroyed, the underlying
@@ -147,7 +155,7 @@ public:
   const char*
   get_path() const;
 
-  std::iostream&
+  std::fstream&
   get_stream();
 
   static temp_file_sptr
@@ -246,6 +254,37 @@ abidiff_status_has_abi_change(abidiff_status s);
 bool
 abidiff_status_has_incompatible_abi_change(abidiff_status s);
 
+/// A type used to time various part of the libabigail system.
+class timer
+{
+  struct priv;
+  typedef shared_ptr<priv> priv_sptr;
+
+  priv_sptr priv_;
+
+public:
+  enum kind
+  {
+    /// Default timer kind.
+    DEFAULT_TIMER_KIND = 0,
+    /// This kind of timer starts upon instantiation.
+    START_ON_INSTANTIATION_TIMER_KIND = 1,
+  };
+
+  timer (kind k = DEFAULT_TIMER_KIND);
+  bool start();
+  bool stop();
+  time_t value_in_seconds() const;
+  bool value(time_t& hours,
+	     time_t& minutes,
+	     time_t& seconds,
+	     time_t& milliseconds) const;
+  string value_as_string() const;
+  ~timer();
+}; //end class timer
+
+ostream& operator<<(ostream&, const timer&);
+
 ostream&
 operator<<(ostream& output, file_type r);
 
@@ -270,7 +309,7 @@ bool
 file_is_kernel_debuginfo_package(const string& file_path,
 				 file_type file_type);
 
-std::tr1::shared_ptr<char>
+abg_compat::shared_ptr<char>
 make_path_absolute(const char*p);
 
 char*
@@ -299,3 +338,5 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
       abort();								\
   } while (false)
 }//end namespace abigail
+
+#endif //__ABG_TOOLS_UTILS_H
