@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -29,6 +29,8 @@
 #ifndef __ABG_WRITER_H__
 #define __ABG_WRITER_H__
 
+#include "abg-fwd.h"
+
 namespace abigail
 {
 namespace xml_writer
@@ -51,17 +53,46 @@ set_show_locs(write_context& ctxt, bool flag);
 void
 set_annotate(write_context& ctxt, bool flag);
 
-bool
-write_translation_unit(const translation_unit&	tu,
-		       unsigned		indent,
-		       std::ostream&		out,
-		       const bool		annotate = false);
+void
+set_write_architecture(write_context& ctxt, bool flag);
+
+void
+set_write_corpus_path(write_context& ctxt, bool flag);
+
+void
+set_write_comp_dir(write_context& ctxt, bool flag);
+
+void
+set_short_locs(write_context& ctxt, bool flag);
+
+/// A convenience generic function to set common options (usually used
+/// by Libabigail tools) from a generic options carrying-object, into
+/// a given @ref write_context.
+///
+/// @param ctxt the @ref the write_context to consider.
+///
+/// @param opts the option-carrying object to set the options from.
+/// It must contain data members named: annotate, and show_locs, at
+/// very least.
+template <typename OPTS>
+void
+set_common_options(write_context& ctxt, const OPTS& opts)
+{
+  set_annotate(ctxt, opts.annotate);
+  set_show_locs(ctxt, opts.show_locs);
+  set_write_architecture(ctxt, opts.write_architecture);
+  set_write_corpus_path(ctxt, opts.write_corpus_path);
+  set_write_comp_dir(ctxt, opts.write_comp_dir);
+  set_short_locs(ctxt, opts.short_locs);
+}
+
+void
+set_ostream(write_context& ctxt, ostream& os);
 
 bool
-write_translation_unit(const translation_unit&	tu,
-		       unsigned		indent,
-		       const string&		path,
-		       const bool		annotate = false);
+write_translation_unit(write_context&	       ctxt,
+		       const translation_unit& tu,
+		       const unsigned	       indent);
 
 bool
 write_corpus_to_archive(const corpus& corp,
@@ -77,38 +108,15 @@ write_corpus_to_archive(const corpus_sptr corp,
 			const bool annotate = false);
 
 bool
-write_corpus(const corpus_sptr	corpus,
+write_corpus(write_context&	ctxt,
+	     const corpus_sptr& corpus,
 	     unsigned		indent,
-	     write_context&	ctxt);
+	     bool		member_of_group = false);
 
 bool
-write_corpus(const corpus_sptr	corpus,
-	     unsigned		indent,
-	     std::ostream&	out,
-	     const bool	annotate = false);
-
-bool
-write_corpus(const corpus_sptr	corpus,
-	     unsigned		indent,
-	     const string&	path,
-	     const bool	annotate = false);
-
-bool
-write_corpus_group(const corpus_group_sptr&	group,
-		   unsigned			indent,
-		   write_context&		ctxt);
-
-bool
-write_corpus_group(const corpus_group_sptr&	group,
-		   unsigned			indent,
-		   std::ostream&		out,
-		   const bool			annotate = false);
-
-bool
-write_corpus_group(const corpus_group_sptr&	group,
-		   unsigned			indent,
-		   const string&		path,
-		   const bool			annotate = false);
+write_corpus_group(write_context&	    ctx,
+		   const corpus_group_sptr& group,
+		   unsigned		    indent);
 
 }// end namespace xml_writer
 }// end namespace abigail
