@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -25,11 +25,13 @@
 /// This file contains definitions of diff objects filtering
 /// facilities.
 
+#include "abg-cxx-compat.h"
 #include "abg-internal.h"
 // <headers defining libabigail's API go under here>
 ABG_BEGIN_EXPORT_DECLARATIONS
 
 #include "abg-comp-filter.h"
+#include "abg-tools-utils.h"
 
 ABG_END_EXPORT_DECLARATIONS
 // </headers defining libabigail's API>
@@ -41,7 +43,7 @@ namespace comparison
 namespace filtering
 {
 
-using std::tr1::dynamic_pointer_cast;
+using abg_compat::dynamic_pointer_cast;
 
 /// Walk the diff sub-trees of a a @ref corpus_diff and apply a filter
 /// to the nodes visted.  The filter categorizes each node, assigning
@@ -174,6 +176,9 @@ has_type_size_change(const diff* diff)
   if (!diff)
     return false;
 
+  if (const fn_parm_diff* fn_parm_d = is_fn_parm_diff(diff))
+    diff = fn_parm_d->type_diff().get();
+
   type_base_sptr f = is_type(diff->first_subject()),
     s = is_type(diff->second_subject());
 
@@ -420,14 +425,24 @@ has_harmless_name_change(const decl_base_sptr& f, const decl_base_sptr& s)
 {
   // So, a harmless name change is either ...
   return (decl_name_changed(f, s)
-	  && (// ... a typedef name change, without having the
+	  && (// ... an anonymous decl name changed into another
+	      // anonymous decl name ...
+	      (f->get_is_anonymous() && s->get_is_anonymous())
+	      ||
+	      // ... an anonymous decl name changed harmlessly into
+	      // another anonymous decl name ...
+	      ((f->get_is_anonymous_or_has_anonymous_parent()
+		&& s->get_is_anonymous_or_has_anonymous_parent())
+	       && tools_utils::decl_names_equal(f->get_qualified_name(),
+						s->get_qualified_name()))
+	      // ... a typedef name change, without having the
 	      // underlying type changed ...
-	      (is_typedef(f)
-	       && is_typedef(s)
-	       && (is_typedef(f)->get_underlying_type()
+	      || (is_typedef(f)
+		  && is_typedef(s)
+		  && (is_typedef(f)->get_underlying_type()
 		   == is_typedef(s)->get_underlying_type()))
-	      // .. or a data member name change, without having the
-	      // its type changed ...
+	      // .. or a data member name change, without having its
+	      // type changed ...
 	      || (is_data_member(f)
 		  && is_data_member(s)
 		  && (is_var_decl(f)->get_type()
@@ -808,6 +823,93 @@ static bool
 base_classes_added_or_removed(const diff* diff)
 {return base_classes_added_or_removed(dynamic_cast<const class_diff*>(diff));}
 
+/// Test if two classes that are decl-only (have the decl-only flag
+/// and carry no data members) but are different just by their size.
+///
+/// In some weird DWARF representation, it happens that a decl-only
+/// class (with no data member) actually carries a non-zero size.
+/// That shouldn't happen, but hey, we need to deal with real life.
+/// So we need to detect that case first.
+///
+/// @param first the first class or union to consider.
+///
+/// @param seconf the second class or union to consider.
+///
+/// @return true if the two classes are decl-only and differ in their
+/// size.
+bool
+is_decl_only_class_with_size_change(const class_or_union& first,
+				    const class_or_union& second)
+{
+  if (first.get_qualified_name() != second.get_qualified_name())
+    return false;
+
+  if (!first.get_is_declaration_only() || !second.get_is_declaration_only())
+    return false;
+
+  bool f_is_empty = first.get_data_members().empty();
+  bool s_is_empty = second.get_data_members().empty();
+
+  return f_is_empty && s_is_empty;
+}
+
+/// Test if two classes that are decl-only (have the decl-only flag
+/// and carry no data members) but are different just by their size.
+///
+/// In some weird DWARF representation, it happens that a decl-only
+/// class (with no data member) actually carries a non-zero size.
+/// That shouldn't happen, but hey, we need to deal with real life.
+/// So we need to detect that case first.
+///
+/// @param first the first class or union to consider.
+///
+/// @param seconf the second class or union to consider.
+///
+/// @return true if the two classes are decl-only and differ in their
+/// size.
+bool
+is_decl_only_class_with_size_change(const class_or_union_sptr& first,
+				    const class_or_union_sptr& second)
+{
+  if (!first || !second)
+    return false;
+
+  class_or_union_sptr f =
+    look_through_decl_only_class(first);
+  class_or_union_sptr s =
+    look_through_decl_only_class(second);
+
+  return is_decl_only_class_with_size_change(*f, *s);
+}
+
+/// Test if a diff node is for two classes that are decl-only (have
+/// the decl-only flag and carry no data members) but are different
+/// just by their size.
+///
+/// In some weird DWARF representation, it happens that a decl-only
+/// class (with no data member) actually carries a non-zero size.
+/// That shouldn't happen, but hey, we need to deal with real life.
+/// So we need to detect that case first.
+///
+/// @param diff the diff node to consider.
+///
+/// @return true if the two classes are decl-only and differ in their
+/// size.
+bool
+is_decl_only_class_with_size_change(const diff *diff)
+{
+  const class_or_union_diff *d = dynamic_cast<const class_or_union_diff*>(diff);
+  if (!d)
+    return false;
+
+  class_or_union_sptr f =
+    look_through_decl_only_class(d->first_class_or_union());
+  class_or_union_sptr s =
+    look_through_decl_only_class(d->second_class_or_union());
+
+  return is_decl_only_class_with_size_change(f, s);
+}
+
 /// Test if two @ref class_or_union_sptr are different just by the
 /// fact that one is decl-only and the other one is defined.
 ///
@@ -1023,6 +1125,52 @@ has_harmful_enum_change(const diff* diff)
   return false;
 }
 
+/// Test if a diff node carries a harmless change of an enum into an
+/// integer (or vice-versa).
+///
+/// The test takes into account the fact change we care about might be
+/// wrapped into a typedef or qualified type diff.
+///
+/// @param diff the diff node to consider.
+///
+/// @return true if @p diff is a harmless enum to integer change.
+static bool
+has_harmless_enum_to_int_change(const diff* diff)
+{
+  if (!diff)
+    return false;
+
+  diff = peel_typedef_or_qualified_type_diff(diff);
+
+  if (const distinct_diff *d = is_distinct_diff(diff))
+    {
+      const enum_type_decl *enum_type = 0;
+      const type_base *integer_type = 0;
+
+      type_base *first_type =
+	peel_qualified_or_typedef_type(is_type(d->first().get()));
+      type_base *second_type =
+	peel_qualified_or_typedef_type(is_type(d->second().get()));
+
+      if (const enum_type_decl *e = is_enum_type(first_type))
+	enum_type = e;
+      else if (const enum_type_decl *e = is_enum_type(second_type))
+	enum_type = e;
+
+      if (const type_base * i = is_type_decl(first_type))
+	integer_type = i;
+      else if (const type_base *i = is_type_decl(second_type))
+	integer_type = i;
+
+      if (enum_type
+	  && integer_type
+	  && enum_type->get_size_in_bits() == integer_type->get_size_in_bits())
+	return true;
+    }
+
+  return false;
+}
+
 /// Test if an @ref fn_parm_diff node has a top cv qualifier change on
 /// the type of the function parameter.
 ///
@@ -1197,7 +1345,7 @@ has_var_type_cv_qual_change(const diff* dif)
 
   {
     // Make sure the variable diff does carry a type change at least
-    change_kind ch_kind;
+    change_kind ch_kind = NO_CHANGE_KIND;
     if (equals(*var_dif->first_var(), *var_dif->second_var(), &ch_kind))
       return false;
 
@@ -1308,6 +1456,25 @@ has_benign_infinite_array_change(const diff* dif)
     }
   return false;
 }
+
+/// Test if a union diff node does have changes that don't impact its
+/// size.
+///
+/// @param d the union diff node to consider.
+///
+/// @return true iff @p d is a diff node which has changes that don't
+/// impact its size.
+bool
+union_diff_has_harmless_changes(const diff *d)
+{
+  if (is_union_diff(d)
+      && d->has_changes()
+      && !has_type_size_change(d))
+    return true;
+
+  return false;
+}
+
 /// Detect if the changes carried by a given diff node are deemed
 /// harmless and do categorize the diff node accordingly.
 ///
@@ -1344,6 +1511,9 @@ categorize_harmless_diff_node(diff *d, bool pre)
 	  || class_diff_has_harmless_odr_violation_change(d))
 	category |= HARMLESS_DECL_NAME_CHANGE_CATEGORY;
 
+      if (union_diff_has_harmless_changes(d))
+	category |= HARMLESS_UNION_CHANGE_CATEGORY;
+
       if (has_non_virtual_mem_fn_change(d))
 	category |= NON_VIRT_MEM_FUN_CHANGE_CATEGORY;
 
@@ -1351,8 +1521,9 @@ categorize_harmless_diff_node(diff *d, bool pre)
 	  || static_data_member_type_size_changed(f, s))
 	category |= STATIC_DATA_MEMBER_CHANGE_CATEGORY;
 
-      if (has_enumerator_insertion(d)
-	  && !has_harmful_enum_change(d))
+      if ((has_enumerator_insertion(d)
+	   && !has_harmful_enum_change(d))
+	  || has_harmless_enum_to_int_change(d))
 	category |= HARMLESS_ENUM_CHANGE_CATEGORY;
 
       if (function_name_changed_but_not_symbol(d))
