@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2016-2019 Red Hat, Inc.
+// Copyright (C) 2016-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -678,6 +678,7 @@ struct corpus::priv
 {
   mutable unordered_map<string, type_base_sptr> canonical_types_;
   environment*					env;
+  corpus_group*				group;
   corpus::exported_decls_builder_sptr		exported_decls_builder;
   origin					origin_;
   vector<string>				regex_patterns_fns_to_suppress;
@@ -717,6 +718,8 @@ struct corpus::priv
   // the type maps of each translation unit.
   type_maps					types_;
   type_maps					type_per_loc_map_;
+  mutable vector<type_base_wptr>		types_not_reachable_from_pub_ifaces_;
+  unordered_set<interned_string, hash_interned_string> *pub_type_pretty_reprs_;
 
 private:
   priv();
@@ -725,8 +728,10 @@ public:
   priv(const string &	p,
        environment*	e)
     : env(e),
+      group(),
       origin_(ARTIFICIAL_ORIGIN),
-      path(p)
+      path(p),
+      pub_type_pretty_reprs_()
   {}
 
   void
@@ -737,6 +742,11 @@ public:
 
   const type_maps&
   get_types() const;
+
+  unordered_set<interned_string, hash_interned_string>*
+  get_public_types_pretty_representations();
+
+  ~priv();
 }; // end struct corpus::priv
 
 void
