@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -24,11 +24,10 @@
 #include <cstdio>
 #include <cstring>
 #include <cassert>
-#include <ext/stdio_filebuf.h>
 #include <stdexcept>
 #include <algorithm>
-#include <tr1/unordered_map>
 
+#include "abg-cxx-compat.h"
 #include "abg-internal.h"
 // <headers defining libabigail's API go under here>
 ABG_BEGIN_EXPORT_DECLARATIONS
@@ -57,7 +56,7 @@ namespace ir
 {
 
 using std::ostringstream;
-using std::tr1::unordered_map;
+using abg_compat::unordered_map;
 using std::list;
 using std::vector;
 
@@ -451,6 +450,29 @@ const type_maps&
 corpus::priv::get_types() const
 {return types_;}
 
+/// Getter of the set of pretty representation of types that are
+/// reachable from public interfaces (global functions and variables).
+///
+/// @return the set of pretty representation of types that are
+/// reachable from public interfaces (global functions and variables).
+unordered_set<interned_string, hash_interned_string>*
+corpus::priv::get_public_types_pretty_representations()
+{
+  if (group)
+    return group->get_public_types_pretty_representations();
+
+  if (pub_type_pretty_reprs_ == 0)
+    pub_type_pretty_reprs_ =
+	new unordered_set<interned_string, hash_interned_string>;
+  return pub_type_pretty_reprs_;
+}
+
+/// Destructor of the @ref corpus::priv type.
+corpus::priv::~priv()
+{
+  delete pub_type_pretty_reprs_;
+}
+
 /// Record a canonical type which has been computed for the current
 /// corpus.
 ///
@@ -611,6 +633,81 @@ const type_maps&
 corpus::get_type_per_loc_map() const
 {return priv_->type_per_loc_map_;}
 
+/// Test if the recording of reachable types (and thus, indirectly,
+/// the recording of non-reachable types) is activated for the
+/// current @ref corpus.
+///
+/// @return true iff the recording of reachable types is activated for
+/// the current @ref corpus.
+bool
+corpus::recording_types_reachable_from_public_interface_supported()
+{
+  return (priv_->get_public_types_pretty_representations()
+	  && !priv_->get_public_types_pretty_representations()->empty());
+}
+
+/// Record a type as being reachable from public interfaces (global
+/// functions and variables).
+///
+/// @param t the type to record as reachable.
+void
+corpus::record_type_as_reachable_from_public_interfaces(const type_base& t)
+{
+  string repr = get_pretty_representation(&t, /*internal=*/true);
+  interned_string s = t.get_environment()->intern(repr);
+  priv_->get_public_types_pretty_representations()->insert(s);
+}
+
+/// Test if a type is reachable from public interfaces (global
+/// functions and variables).
+///
+/// For a type to be considered reachable from public interfaces, it
+/// must have been previously marked as such by calling
+/// corpus::record_type_as_reachable_from_public_interfaces.
+///
+/// @param t the type to test for.
+///
+/// @return true iff @p t is reachable from public interfaces.
+bool
+corpus::type_is_reachable_from_public_interfaces(const type_base& t) const
+{
+  string repr = get_pretty_representation(&t, /*internal=*/true);
+  interned_string s = t.get_environment()->intern(repr);
+
+  return (priv_->get_public_types_pretty_representations()->find(s)
+	  !=  priv_->get_public_types_pretty_representations()->end());
+}
+
+/// Getter of a sorted vector of the types that are *NOT* reachable
+/// from public interfaces.
+///
+/// Note that for this to be non-empty, the libabigail reader that
+/// analyzed the input (be it a binary or an abixml file) must have be
+/// configured to load types that are not reachable from public
+/// interfaces.
+///
+/// @return a reference to a vector of sorted types NON reachable from
+/// public interfaces.
+const vector<type_base_wptr>&
+corpus::get_types_not_reachable_from_public_interfaces() const
+{
+  if (priv_->types_not_reachable_from_pub_ifaces_.empty())
+    {
+      const type_maps& types = get_types();
+      for (vector<type_base_wptr>::const_iterator it =
+	     types.get_types_sorted_by_name().begin();
+	   it != types.get_types_sorted_by_name().end();
+	   ++it)
+	{
+	  type_base_sptr t(*it);
+	  if (!type_is_reachable_from_public_interfaces(*t))
+	    priv_->types_not_reachable_from_pub_ifaces_.push_back(t);
+	}
+    }
+
+  return priv_->types_not_reachable_from_pub_ifaces_;
+}
+
 /// Get the maps that associate a location string to a certain kind of
 /// type.
 ///
@@ -624,6 +721,29 @@ type_maps&
 corpus::get_type_per_loc_map()
 {return priv_->type_per_loc_map_;}
 
+/// Getter of the group this corpus is a member of.
+///
+/// @return the group this corpus is a member of, or nil if it's not
+/// part of any @ref corpus_group.
+const corpus_group*
+corpus::get_group() const
+{return priv_->group;}
+
+/// Getter of the group this corpus belongs to.
+///
+/// @return the group this corpus belong to, or nil if it's not part
+/// of any @ref corpus_group.
+corpus_group*
+corpus::get_group()
+{return priv_->group;}
+
+/// Setter of the group this corpus belongs to.
+///
+/// @param g the new group.
+void
+corpus::set_group(corpus_group* g)
+{priv_->group = g;}
+
 /// Getter for the origin of the corpus.
 ///
 /// @return the origin of the corpus.
@@ -729,13 +849,30 @@ void
 corpus::set_architecture_name(const string& arch)
 {priv_->architecture_name = arch;}
 
-/// Tests if the corpus contains no translation unit.
+/// Tests if the corpus is empty from an ABI surface perspective. I.e. if all
+/// of these criteria are true:
+///  - all translation units (members) are empty
+///  - the maps function and variable symbols are not having entries
+///  - for shared libraries:
+///    - the soname is empty
+///    - there are no DT_NEEDED entries
 ///
 /// @return true if the corpus contains no translation unit.
 bool
 corpus::is_empty() const
 {
-  return (priv_->members.empty()
+  bool members_empty = true;
+  for (translation_units::const_iterator i = priv_->members.begin(),
+					 e = priv_->members.end();
+       i != e; ++i)
+    {
+      if (!(*i)->is_empty())
+	{
+	  members_empty = false;
+	  break;
+	}
+    }
+  return (members_empty
 	  && priv_->fun_symbol_map
 	  && priv_->fun_symbol_map->empty()
 	  && priv_->var_symbol_map
@@ -1043,12 +1180,12 @@ corpus::get_sorted_undefined_var_symbols() const
 const elf_symbol_sptr
 corpus::lookup_function_symbol(const string& n) const
 {
-  if (!get_fun_symbol_map_sptr())
+  if (get_fun_symbol_map().empty())
     return elf_symbol_sptr();
 
   string_elf_symbols_map_type::const_iterator it =
-    get_fun_symbol_map_sptr()->find(n);
-  if ( it == get_fun_symbol_map_sptr()->end())
+    get_fun_symbol_map().find(n);
+  if ( it == get_fun_symbol_map().end())
     return elf_symbol_sptr();
   return it->second[0];
 }
@@ -1110,12 +1247,12 @@ const elf_symbol_sptr
 corpus::lookup_function_symbol(const string& symbol_name,
 			       const elf_symbol::version& version) const
 {
-  if (!get_fun_symbol_map_sptr())
+  if (get_fun_symbol_map().empty())
     return elf_symbol_sptr();
 
   string_elf_symbols_map_type::const_iterator it =
-    get_fun_symbol_map_sptr()->find(symbol_name);
-  if ( it == get_fun_symbol_map_sptr()->end())
+    get_fun_symbol_map().find(symbol_name);
+  if ( it == get_fun_symbol_map().end())
     return elf_symbol_sptr();
 
   return find_symbol_by_version(version, it->second);
@@ -1139,12 +1276,12 @@ corpus::lookup_function_symbol(const elf_symbol& symbol) const
 const elf_symbol_sptr
 corpus::lookup_variable_symbol(const string& n) const
 {
-    if (!get_var_symbol_map_sptr())
+  if (get_var_symbol_map().empty())
     return elf_symbol_sptr();
 
   string_elf_symbols_map_type::const_iterator it =
-    get_var_symbol_map_sptr()->find(n);
-  if ( it == get_var_symbol_map_sptr()->end())
+    get_var_symbol_map().find(n);
+  if ( it == get_var_symbol_map().end())
     return elf_symbol_sptr();
   return it->second[0];
 }
@@ -1161,12 +1298,12 @@ const elf_symbol_sptr
 corpus::lookup_variable_symbol(const string& symbol_name,
 			       const elf_symbol::version& version) const
 {
-    if (!get_var_symbol_map_sptr())
+  if (get_var_symbol_map().empty())
     return elf_symbol_sptr();
 
   string_elf_symbols_map_type::const_iterator it =
-    get_var_symbol_map_sptr()->find(symbol_name);
-  if ( it == get_var_symbol_map_sptr()->end())
+    get_var_symbol_map().find(symbol_name);
+  if ( it == get_var_symbol_map().end())
     return elf_symbol_sptr();
 
   return find_symbol_by_version(version, it->second);
@@ -1502,6 +1639,7 @@ struct corpus_group::priv
   unordered_map<string, elf_symbol_sptr> unrefed_var_symbol_map;
   elf_symbols			unrefed_var_symbols;
   bool				unrefed_var_symbols_built;
+  unordered_set<interned_string, hash_interned_string> pub_type_pretty_reprs_;
 
   priv()
     : unrefed_fun_symbols_built(),
@@ -1589,6 +1727,7 @@ corpus_group::add_corpus(const corpus_sptr& corp)
     ABG_ASSERT(cur_arch == corp_arch);
 
   priv_->corpora.push_back(corp);
+  corp->set_group(this);
 
   /// Add the unreferenced function and variable symbols of this
   /// corpus to the unreferenced symbols of the current corpus group.
@@ -1604,6 +1743,24 @@ const corpus_group::corpora_type&
 corpus_group::get_corpora() const
 {return priv_->corpora;}
 
+/// Getter of the first corpus added to this Group.
+///
+/// @return the first corpus added to this Group.
+const corpus_sptr
+corpus_group::get_main_corpus() const
+{return const_cast<corpus_group*>(this)->get_main_corpus();}
+
+/// Getter of the first corpus added to this Group.
+///
+/// @return the first corpus added to this Group.
+corpus_sptr
+corpus_group::get_main_corpus()
+{
+  if (!get_corpora().empty())
+    return get_corpora().front();
+  return corpus_sptr();
+}
+
 /// Test if the current corpus group is empty.
 ///
 /// @return true iff the current corpus group is empty.
@@ -1872,6 +2029,22 @@ corpus_group::get_unreferenced_variable_symbols() const
   return priv_->unrefed_var_symbols;
 }
 
+/// Getter of a pointer to the set of types reachable from public
+/// interfaces of a given corpus group.
+unordered_set<interned_string, hash_interned_string>*
+corpus_group::get_public_types_pretty_representations()
+{return &priv_->pub_type_pretty_reprs_;}
+
+/// Test if the recording of reachable types (and thus, indirectly,
+/// the recording of non-reachable types) is activated for the
+/// current @ref corpus_group.
+///
+/// @return true iff the recording of reachable types is activated for
+/// the current @ref corpus_group.
+bool
+corpus_group::recording_types_reachable_from_public_interface_supported()
+{return !get_public_types_pretty_representations()->empty();}
+
 // </corpus_group stuff>
 
 }// end namespace ir
