@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2017-2019 Red Hat, Inc.
+// Copyright (C) 2017-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -1455,6 +1455,23 @@ default_reporter::report(const union_diff& d, ostream& out,
 
   d.class_or_union_diff::report(out, indent);
 
+  if (d.context()->get_allowed_category() & HARMLESS_UNION_CHANGE_CATEGORY
+      && filtering::union_diff_has_harmless_changes(&d))
+    {
+      // The user wants to see harmless changes and the union diff we
+      // are looking at does carry some harmless changes.  Let's show
+      // the "before" and "after" carried by the diff node.
+      out << indent << "type changed from:\n"
+	  << get_class_or_union_flat_representation(first, indent + "  ",
+						    /*one_line=*/true,
+						    /*qualified_names=*/false)
+	  << "\n"
+	  << indent << "to:\n"
+	  << get_class_or_union_flat_representation(second, indent + "  ",
+						    /*one_line=*/true,
+						    /*qualified_names=*/false);
+    }
+
  d.currently_reporting(false);
 
   d.reported_once(true);
@@ -2048,7 +2065,8 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 
   if (ctxt->show_changed_vars())
     {
-      size_t num_changed = s.num_vars_changed() - s.num_changed_vars_filtered_out();
+      size_t num_changed =
+	s.num_vars_changed() - s.num_changed_vars_filtered_out();
       if (num_changed == 1)
 	out << indent << "1 Changed variable:\n\n";
       else if (num_changed > 1)
@@ -2227,6 +2245,11 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	out << '\n';
     }
 
+  // Report added/removed/changed types not reacheable from public
+  // interfaces.
+
+  maybe_report_unreachable_type_changes(d, s, indent, out);
+
   d.priv_->maybe_dump_diff_tree();
 }
 
