@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -20,6 +20,7 @@
 
 /// @file
 
+#include "abg-cxx-compat.h"
 #include "abg-internal.h"
 // <headers defining libabigail's API go under here>
 ABG_BEGIN_EXPORT_DECLARATIONS
@@ -78,8 +79,8 @@ using namespace abigail::ir;
 size_t
 type_base::hash::operator()(const type_base& t) const
 {
-  std::tr1::hash<size_t> size_t_hash;
-  std::tr1::hash<string> str_hash;
+  abg_compat::hash<size_t> size_t_hash;
+  abg_compat::hash<string> str_hash;
 
   size_t v = str_hash(typeid(t).name());
   v = hashing::combine_hashes(v, size_t_hash(t.get_size_in_bits()));
@@ -111,7 +112,7 @@ struct decl_base::hash
   size_t
   operator()(const decl_base& d) const
   {
-    std::tr1::hash<string> str_hash;
+    abg_compat::hash<string> str_hash;
 
     size_t v = str_hash(typeid(d).name());
     if (!d.get_linkage_name().empty())
@@ -134,7 +135,7 @@ struct type_decl::hash
   {
     decl_base::hash decl_hash;
     type_base::hash type_hash;
-    std::tr1::hash<string> str_hash;
+    abg_compat::hash<string> str_hash;
 
     size_t v = str_hash(typeid(t).name());
     v = hashing::combine_hashes(v, decl_hash(t));
@@ -152,7 +153,7 @@ struct type_decl::hash
 size_t
 scope_decl::hash::operator()(const scope_decl& d) const
 {
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   size_t v = hash_string(typeid(d).name());
   for (scope_decl::declarations::const_iterator i =
 	 d.get_member_decls().begin();
@@ -179,7 +180,7 @@ struct scope_type_decl::hash
   {
     decl_base::hash decl_hash;
     type_base::hash type_hash;
-    std::tr1::hash<string> str_hash;
+    abg_compat::hash<string> str_hash;
 
     size_t v = str_hash(typeid(t).name());
     v = hashing::combine_hashes(v, decl_hash(t));
@@ -196,7 +197,7 @@ struct qualified_type_def::hash
   {
     type_base::hash type_hash;
     decl_base::hash decl_hash;
-    std::tr1::hash<string> str_hash;
+    abg_compat::hash<string> str_hash;
 
     size_t v = str_hash(typeid(t).name());
     v = hashing::combine_hashes(v, type_hash(t));
@@ -211,7 +212,7 @@ struct pointer_type_def::hash
   size_t
   operator()(const pointer_type_def& t) const
   {
-    std::tr1::hash<string> str_hash;
+    abg_compat::hash<string> str_hash;
     type_base::hash type_base_hash;
     decl_base::hash decl_hash;
     type_base::shared_ptr_hash hash_type_ptr;
@@ -229,7 +230,7 @@ struct reference_type_def::hash
   size_t
   operator()(const reference_type_def& t)
   {
-    std::tr1::hash<string> hash_str;
+    abg_compat::hash<string> hash_str;
     type_base::hash hash_type_base;
     decl_base::hash hash_decl;
     type_base::shared_ptr_hash hash_type_ptr;
@@ -250,7 +251,7 @@ struct array_type_def::subrange_type::hash
   size_t
   operator()(const array_type_def::subrange_type& s) const
   {
-    std::tr1::hash<int> hash_size_t;
+    abg_compat::hash<int> hash_size_t;
     size_t v = hash_size_t(hash_size_t(s.get_lower_bound()));
     v = hashing::combine_hashes(v, hash_size_t(s.get_upper_bound()));
     return v;
@@ -262,7 +263,7 @@ struct array_type_def::hash
   size_t
   operator()(const array_type_def& t)
   {
-    std::tr1::hash<string> hash_str;
+    abg_compat::hash<string> hash_str;
     type_base::hash hash_type_base;
     decl_base::hash hash_decl;
     type_base::shared_ptr_hash hash_type_ptr;
@@ -289,10 +290,10 @@ struct enum_type_decl::hash
   size_t
   operator()(const enum_type_decl& t) const
   {
-    std::tr1::hash<string> str_hash;
+    abg_compat::hash<string> str_hash;
     decl_base::hash decl_hash;
     type_base::shared_ptr_hash type_ptr_hash;
-    std::tr1::hash<size_t> size_t_hash;
+    abg_compat::hash<size_t> size_t_hash;
 
     size_t v = str_hash(typeid(t).name());
     v = hashing::combine_hashes(v, decl_hash(t));
@@ -314,7 +315,7 @@ struct typedef_decl::hash
   size_t
   operator()(const typedef_decl& t) const
   {
-    std::tr1::hash<string> str_hash;
+    abg_compat::hash<string> str_hash;
     type_base::hash hash_type;
     decl_base::hash decl_hash;
     type_base::shared_ptr_hash type_ptr_hash;
@@ -340,10 +341,10 @@ struct typedef_decl::hash
 size_t
 var_decl::hash::operator()(const var_decl& t) const
 {
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   decl_base::hash hash_decl;
   type_base::shared_ptr_hash hash_type_ptr;
-  std::tr1::hash<size_t> hash_size_t;
+  abg_compat::hash<size_t> hash_size_t;
 
   size_t v = hash_string(typeid(t).name());
   v = hashing::combine_hashes(v, hash_decl(t));
@@ -379,10 +380,10 @@ var_decl::hash::operator()(const var_decl* t) const
 size_t
 function_decl::hash::operator()(const function_decl& t) const
 {
-  std::tr1::hash<int> hash_int;
-  std::tr1::hash<size_t> hash_size_t;
-  std::tr1::hash<bool> hash_bool;
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<int> hash_int;
+  abg_compat::hash<size_t> hash_size_t;
+  abg_compat::hash<bool> hash_bool;
+  abg_compat::hash<string> hash_string;
   decl_base::hash hash_decl_base;
   type_base::shared_ptr_hash hash_type_ptr;
 
@@ -424,8 +425,8 @@ function_decl::parameter::hash::operator()
   (const function_decl::parameter& p) const
 {
   type_base::shared_ptr_hash hash_type_ptr;
-  std::tr1::hash<bool> hash_bool;
-  std::tr1::hash<unsigned> hash_unsigned;
+  abg_compat::hash<bool> hash_bool;
+  abg_compat::hash<unsigned> hash_unsigned;
   size_t v = hash_type_ptr(p.get_type());
   v = hashing::combine_hashes(v, hash_unsigned(p.get_index()));
   v = hashing::combine_hashes(v, hash_bool(p.get_variadic_marker()));
@@ -448,7 +449,7 @@ struct method_type::hash
   size_t
   operator()(const method_type& t) const
   {
-    std::tr1::hash<string> hash_string;
+    abg_compat::hash<string> hash_string;
     type_base::shared_ptr_hash hash_type_ptr;
     function_decl::parameter::hash hash_parameter;
 
@@ -484,7 +485,7 @@ struct method_type::hash
 size_t
 function_type::hash::operator()(const function_type& t) const
 {
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   type_base::shared_ptr_hash hash_type_ptr;
   function_decl::parameter::hash hash_parameter;
 
@@ -528,7 +529,7 @@ function_type::hash::operator()(const function_type_sptr t) const
 size_t
 member_base::hash::operator()(const member_base& m) const
 {
-  std::tr1::hash<int> hash_int;
+  abg_compat::hash<int> hash_int;
   return hash_int(m.get_access_specifier());
 }
 
@@ -537,9 +538,9 @@ class_decl::base_spec::hash::operator()(const base_spec& t) const
 {
   member_base::hash hash_member;
   type_base::shared_ptr_hash hash_type_ptr;
-  std::tr1::hash<size_t> hash_size;
-  std::tr1::hash<bool> hash_bool;
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<size_t> hash_size;
+  abg_compat::hash<bool> hash_bool;
+  abg_compat::hash<string> hash_string;
 
   size_t v = hash_string(typeid(t).name());
   v = hashing::combine_hashes(v, hash_member(t));
@@ -553,10 +554,10 @@ size_t
 member_function_template::hash::operator()
   (const member_function_template& t) const
 {
-  std::tr1::hash<bool> hash_bool;
+  abg_compat::hash<bool> hash_bool;
   function_tdecl::hash hash_function_tdecl;
   member_base::hash hash_member;
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
 
   size_t v = hash_member(t);
   string n = t.get_qualified_name();
@@ -573,7 +574,7 @@ member_class_template::hash::operator()
 {
   member_base::hash hash_member;
   class_tdecl::hash hash_class_tdecl;
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
 
   size_t v = hash_member(t);
   string n = t.get_qualified_name();
@@ -610,7 +611,7 @@ class_or_union::hash::operator()(const class_or_union& t) const
 
   ABG_ASSERT(!t.get_is_declaration_only());
 
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   scope_type_decl::hash hash_scope_type;
   var_decl::hash hash_data_member;
   member_function_template::hash hash_member_fn_tmpl;
@@ -689,7 +690,7 @@ class_decl::hash::operator()(const class_decl& t) const
 
   ABG_ASSERT(!t.get_is_declaration_only());
 
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   class_decl::base_spec::hash hash_base;
   class_or_union::hash hash_class_or_union;
 
@@ -738,8 +739,8 @@ struct template_parameter::hash
 
     t.set_hashing_has_started(true);
 
-    std::tr1::hash<unsigned> hash_unsigned;
-    std::tr1::hash<std::string> hash_string;
+    abg_compat::hash<unsigned> hash_unsigned;
+    abg_compat::hash<std::string> hash_string;
     template_decl::hash hash_template_decl;
 
     size_t v = hash_string(typeid(t).name());
@@ -769,7 +770,7 @@ struct template_parameter::shared_ptr_hash
 size_t
 template_decl::hash::operator()(const template_decl& t) const
 {
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   template_parameter::shared_ptr_hash hash_template_parameter;
 
   size_t v = hash_string(typeid(t).name());
@@ -790,7 +791,7 @@ struct type_tparameter::hash
   size_t
   operator()(const type_tparameter& t) const
   {
-    std::tr1::hash<string> hash_string;
+    abg_compat::hash<string> hash_string;
     template_parameter::hash hash_template_parameter;
     type_decl::hash hash_type;
 
@@ -811,7 +812,7 @@ size_t
 non_type_tparameter::hash::operator()(const non_type_tparameter& t) const
 {
   template_parameter::hash hash_template_parameter;
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   type_base::shared_ptr_hash hash_type;
 
   size_t v = hash_string(typeid(t).name());
@@ -836,7 +837,7 @@ struct template_tparameter::hash
   size_t
   operator()(const template_tparameter& t) const
   {
-    std::tr1::hash<string> hash_string;
+    abg_compat::hash<string> hash_string;
     type_tparameter::hash hash_template_type_parm;
     template_decl::hash hash_template_decl;
 
@@ -874,7 +875,7 @@ operator()(const template_parameter* t) const
 size_t
 type_composition::hash::operator()(const type_composition& t) const
 {
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   type_base::dynamic_hash hash_type;
 
   size_t v = hash_string(typeid(t).name());
@@ -895,7 +896,7 @@ size_t
 function_tdecl::hash::
 operator()(const function_tdecl& t) const
 {
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   decl_base::hash hash_decl_base;
   template_decl::hash hash_template_decl;
   function_decl::hash hash_function_decl;
@@ -924,7 +925,7 @@ size_t
 class_tdecl::hash::
 operator()(const class_tdecl& t) const
 {
-  std::tr1::hash<string> hash_string;
+  abg_compat::hash<string> hash_string;
   decl_base::hash hash_decl_base;
   template_decl::hash hash_template_decl;
   class_decl::hash hash_class_decl;
