@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2017-2019 Red Hat, Inc.
+// Copyright (C) 2017-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -1231,6 +1231,10 @@ leaf_reporter::report(const corpus_diff& d,
   // Now show the changed types.
   const diff_maps& leaf_diffs = d.get_leaf_diffs();
   report_type_changes_from_diff_maps(*this, leaf_diffs, out, indent);
+
+  // Report added/removed/changed types not reacheable from public
+  // interfaces.
+  maybe_report_unreachable_type_changes(d, s, indent, out);
 }
 } // end namespace comparison
 } // end namespace abigail
