@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2017-2019 Red Hat, Inc.
+// Copyright (C) 2017-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -421,6 +421,7 @@ represent(const var_diff_sptr	&diff,
       is_strict_anonymous_data_member_change = true;
       string tr1 = o->get_pretty_representation();
       string tr2 = n->get_pretty_representation();
+      type_base_sptr t1 = o->get_type(), t2 = n->get_type();
       if (tr1 != tr2)
 	{
 	  show_offset_or_size(indent + "anonymous data member at offset",
@@ -432,6 +433,22 @@ represent(const var_diff_sptr	&diff,
 	      << indent << "to:\n"
 	      << indent << "  " << tr2 << "\n";
 	}
+      else if (get_type_name(t1) != get_type_name(t2)
+	       && is_decl(t1) && is_decl(t2)
+	       && is_decl(t1)->get_is_anonymous()
+	       && is_decl(t2)->get_is_anonymous())
+	{
+	  out << indent << "while looking at anonymous data member '"
+	      << tr1 << "':\n"
+	      << indent << "the internal name of that anonymous data member"
+	                   "changed from:\n"
+	      << indent << " " << get_type_name(o->get_type()) << "\n"
+	      << indent << "to:\n"
+	      << indent << " " << get_type_name(n->get_type()) << "\n"
+	      << indent << " This is usually due to "
+	      <<"an anonymous member type being added or removed from "
+	      << "the containing type\n";
+	}
     }
   else if (filtering::has_anonymous_data_member_change(diff))
     {
@@ -469,30 +486,25 @@ represent(const var_diff_sptr	&diff,
     {
       if (local_only && d->has_local_changes())
 	{
-	  std::ostringstream out_buffer;
-	    out_buffer
-	      << indent << "type '" << get_pretty_representation(o->get_type())
+	  out << indent << "type '" << get_pretty_representation(o->get_type())
 	      << "' of '" << o->get_qualified_name()
 	      << "' changed";
 
 	  if (d->currently_reporting())
 	    {
-	      out_buffer << " as being reported\n";
-	      begin_with_and = true;
+	      out << " as being reported\n";
 	    }
 	  else if (d->reported_once())
 	    {
-	      out_buffer << " as reported earlier\n";
-	      begin_with_and = true;
+	      out << " as reported earlier\n";
 	    }
 	  else
 	    {
-	      out_buffer << ":\n";
-	      d->report(out_buffer, indent + "  ");
-	      begin_with_and = false;
+	      out << ":\n";
+	      d->report(out, indent + "  ");
 	    }
 
-	  out << out_buffer.str();
+	  begin_with_and = true;
 	  emitted = true;
 	}
       else
@@ -1202,8 +1214,148 @@ show_linkage_name_and_aliases(ostream& out,
     out << ", aliases " << aliases;
 }
 
+/// Report changes about types that are not reachable from global
+/// functions and variables, in a given @param corpus_diff.
+///
+/// @param d the corpus_diff to consider.
+///
+/// @param s the statistics of the changes, after filters and
+/// suppressions are reported.  This is typically what is returned by
+/// corpus_diff::apply_filters_and_suppressions_before_reporting().
+///
+/// @param indent the indendation string (usually a string of white
+/// spaces) to use for indentation during the reporting.
+///
+/// @param out the output stream to emit the report to.
+void
+maybe_report_unreachable_type_changes(const corpus_diff& d,
+				      const corpus_diff::diff_stats &s,
+				      const string& indent,
+				      ostream& out)
+{
+  const unsigned large_num = 100;
+  const diff_context_sptr& ctxt = d.context();
+
+  if (!(ctxt->show_unreachable_types()
+	&& (!d.priv_->deleted_unreachable_types_.empty()
+	    || !d.priv_->added_unreachable_types_.empty()
+	    || !d.priv_->changed_unreachable_types_.empty())))
+    // The user either doesn't want us to show changes about
+    // unreachable types or there are not such changes.
+    return;
+
+  // Handle removed unreachable types.
+  if (s.net_num_removed_unreachable_types() == 1)
+    out << indent
+	<< "1 removed type unreachable from any public interface:\n\n";
+  else if (s.net_num_removed_unreachable_types() > 1)
+    out << indent
+	<< s.net_num_removed_unreachable_types()
+	<< " removed types unreachable from any public interface:\n\n";
+
+  vector<type_base_sptr> sorted_removed_unreachable_types;
+  sort_string_type_base_sptr_map(d.priv_->deleted_unreachable_types_,
+				 sorted_removed_unreachable_types);
+  bool emitted = false;
+  for (vector<type_base_sptr>::const_iterator i =
+	 sorted_removed_unreachable_types.begin();
+       i != sorted_removed_unreachable_types.end();
+       ++i)
+    {
+      if (d.priv_->deleted_unreachable_type_is_suppressed((*i).get()))
+	continue;
+
+      out << indent << "  ";
+      if (s.num_removed_unreachable_types() > large_num)
+	out << "[D] ";
+      out << get_pretty_representation(*i);
+      report_loc_info(*i, *ctxt, out);
+      out << "\n";
+      emitted = true;
+    }
+  if (emitted)
+    {
+      out << "\n";
+      emitted = false;
+    }
+
+  // Handle changed unreachable types!
+  if (s.net_num_changed_unreachable_types() == 1)
+    out << indent
+	<< "1 changed type unreachable from any public interface:\n\n";
+  else if (s.net_num_changed_unreachable_types() > 1)
+    out << indent
+	<< s.net_num_changed_unreachable_types()
+	<< " changed types unreachable from any public interface:\n\n";
+
+  diff_sptrs_type sorted_diff_sptrs;
+  sort_string_diff_sptr_map(d.priv_->changed_unreachable_types_,
+			    sorted_diff_sptrs);
+  emitted =  true;
+  for (diff_sptrs_type::const_iterator i = sorted_diff_sptrs.begin();
+       i != sorted_diff_sptrs.end();
+       ++i)
+    {
+      diff_sptr diff = *i;
+      if (!diff || !diff->to_be_reported())
+	continue;
+
+      string repr = diff->first_subject()->get_pretty_representation();
+
+      out << "  ";
+
+      if (sorted_diff_sptrs.size() > large_num)
+	out << "[C] ";
+
+      out << "'" << repr << "' changed:\n";
+      diff->report(out, indent + "    ");
+      out << "\n";
+      emitted = true;
+    }
+  if (emitted)
+    {
+      out << "\n";
+      emitted = false;
+    }
+
+  // Handle added unreachable types.
+  if (s.net_num_added_unreachable_types() == 1)
+    out << indent
+	<< "1 added type unreachable from any public interface:\n\n";
+  else if (s.net_num_added_unreachable_types() > 1)
+    out << indent
+	<< s.net_num_added_unreachable_types()
+	<< " added types unreachable from any public interface:\n\n";
+
+  vector<type_base_sptr> sorted_added_unreachable_types;
+  sort_string_type_base_sptr_map(d.priv_->added_unreachable_types_,
+				 sorted_added_unreachable_types);
+  emitted = false;
+  for (vector<type_base_sptr>::const_iterator i =
+	 sorted_added_unreachable_types.begin();
+       i != sorted_added_unreachable_types.end();
+       ++i)
+    {
+      if (d.priv_->added_unreachable_type_is_suppressed((*i).get()))
+	continue;
+
+      out << indent << "  ";
+      if (s.num_added_unreachable_types() > large_num)
+	out << "[A] ";
+      out << "'" << get_pretty_representation(*i) << "'";
+      report_loc_info(*i, *ctxt, out);
+      out << "\n";
+      emitted = true;
+    }
+  if (emitted)
+    {
+      out << "\n";
+      emitted = false;
+    }
+}
+
 /// If a given diff node impacts some public interfaces, then report
-/// about those impacted interfaces on standard output.
+/// about those impacted interfaces on a given output stream.
 ///
 /// @param d the diff node to get the impacted interfaces for.
 ///
