@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -20,12 +20,12 @@
 //
 // Author: Dodji Seketeli
 
-#include "abg-comparison.h"
-#include "abg-reporter.h"
-
 #ifndef __ABG_REPORTER_PRIV_H__
 #define __ABG_REPORTER_PRIV_H__
 
+#include "abg-comparison.h"
+#include "abg-reporter.h"
+
 /// This is a subroutine of a *::report() function.
 ///
 /// If the diff about two subjects S1 and S2 was reported earlier or
@@ -235,6 +235,12 @@ show_linkage_name_and_aliases(ostream& out,
 			      const elf_symbol& symbol,
 			      const string_elf_symbols_map_type& sym_map);
 
+void
+maybe_report_unreachable_type_changes(const corpus_diff& d,
+				      const corpus_diff::diff_stats &s,
+				      const string& indent,
+				      ostream& out);
+
 void
 maybe_report_interfaces_impacted_by_diff(const diff		*d,
 					 ostream		&out,
