@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2016-2019 Red Hat, Inc.
+// Copyright (C) 2016-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -179,32 +179,72 @@ public:
     return soname_not_regex_;
   }
 
+  /// Test if the current suppression matches a given SONAME.
+  ///
+  /// @param soname the SONAME to consider.
+  ///
+  /// @return true iff the suppression matches the SONAME denoted by
+  /// @p soname.
+  ///
+  /// Note that if the suppression contains no property that is
+  /// related to SONAMEs, the function returns false.
   bool
   matches_soname(const string& soname) const
   {
+    bool has_regexp = false;
     if (sptr_utils::regex_t_sptr regexp = get_soname_regex())
-      if (regexec(regexp.get(), soname.c_str(), 0, NULL, 0) != 0)
-	return false;
+      {
+	has_regexp = true;
+	if (regexec(regexp.get(), soname.c_str(), 0, NULL, 0) != 0)
+	  return false;
+      }
 
     if (sptr_utils::regex_t_sptr regexp = get_soname_not_regex())
-      if (regexec(regexp.get(), soname.c_str(), 0, NULL, 0) == 0)
+      {
+	has_regexp = true;
+	if (regexec(regexp.get(), soname.c_str(), 0, NULL, 0) == 0)
+	  return false;
+      }
+
+      if (!has_regexp)
 	return false;
 
     return true;
   }
 
+  /// Test if the current suppression matches the full file path to a
+  /// given binary.
+  ///
+  /// @param binary_name the full path to the binary.
+  ///
+  /// @return true iff the suppression matches the path denoted by @p
+  /// binary_name.
+  ///
+  /// Note that if the suppression contains no property that is
+  /// related to file name, the function returns false.
   bool
   matches_binary_name(const string& binary_name) const
   {
+    bool has_regexp = false;
+
     if (sptr_utils::regex_t_sptr regexp = get_file_name_regex())
-      if (regexec(regexp.get(), binary_name.c_str(),
-		  0, NULL, 0) != 0)
-	return false;
+      {
+	has_regexp = true;
+	if (regexec(regexp.get(), binary_name.c_str(),
+	  0, NULL, 0) != 0)
+	  return false;
+      }
 
     if (sptr_utils::regex_t_sptr regexp = get_file_name_not_regex())
-      if (regexec(regexp.get(), binary_name.c_str(),
-		  0, NULL, 0) == 0)
-	return false;
+      {
+	has_regexp = true;
+	if (regexec(regexp.get(), binary_name.c_str(),
+	  0, NULL, 0) == 0)
+	  return false;
+      }
+
+    if (!has_regexp)
+      return false;
 
     return true;
   }
@@ -272,6 +312,8 @@ struct function_suppression::priv
   string				symbol_name_;
   string				symbol_name_regex_str_;
   mutable sptr_utils::regex_t_sptr	symbol_name_regex_;
+  string				symbol_name_not_regex_str_;
+  mutable sptr_utils::regex_t_sptr	symbol_name_not_regex_;
   string				symbol_version_;
   string				symbol_version_regex_str_;
   mutable sptr_utils::regex_t_sptr	symbol_version_regex_;
@@ -397,6 +439,29 @@ struct function_suppression::priv
     return symbol_name_regex_;
   }
 
+  /// Getter for a pointer to a regular expression object built from
+  /// the regular expression string
+  /// function_suppression::priv::symbol_name_not_regex_str_.
+  ///
+  /// If that string is empty, then an empty regular expression object
+  /// pointer is returned.
+  ///
+  /// @return a pointer to the regular expression object of
+  /// function_suppression::priv::symbol_name_not_regex_str_.
+  const sptr_utils::regex_t_sptr
+  get_symbol_name_not_regex() const
+  {
+    if (!symbol_name_not_regex_ && !symbol_name_not_regex_str_.empty())
+      {
+	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
+	if (regcomp(r.get(),
+		    symbol_name_not_regex_str_.c_str(),
+		    REG_EXTENDED) == 0)
+	  symbol_name_not_regex_ = r;
+      }
+    return symbol_name_not_regex_;
+  }
+
   /// Getter for a pointer to a regular expression object built from
   /// the regular expression string
   /// function_suppression::priv::symbol_version_regex_str_.
@@ -474,10 +539,12 @@ function_is_suppressed(const ReadContextType&	ctxt,
       {
 	if (require_drop_property && !(*i)->get_drops_artifact_from_ir())
 	  continue;
-	if (ctxt.suppression_matches_function_name(*suppr, fn_name))
+	if (!fn_name.empty()
+	    && ctxt.suppression_matches_function_name(*suppr, fn_name))
 	  return true;
-	if (ctxt.suppression_matches_function_sym_name(*suppr,
-						       fn_linkage_name))
+	if (!fn_linkage_name.empty()
+	    && ctxt.suppression_matches_function_sym_name(*suppr,
+							  fn_linkage_name))
 	  return true;
       }
   return false;
@@ -500,6 +567,8 @@ struct variable_suppression::priv
   string				symbol_name_;
   string				symbol_name_regex_str_;
   mutable sptr_utils::regex_t_sptr	symbol_name_regex_;
+  string				symbol_name_not_regex_str_;
+  mutable sptr_utils::regex_t_sptr	symbol_name_not_regex_;
   string				symbol_version_;
   string				symbol_version_regex_str_;
   mutable sptr_utils::regex_t_sptr	symbol_version_regex_;
@@ -595,6 +664,28 @@ struct variable_suppression::priv
     return symbol_name_regex_;
   }
 
+  /// Getter for a pointer to a regular expression object built from
+  /// the regular expression string
+  /// variable_suppression::priv::symbol_name_not_regex_str_.
+  ///
+  /// If that string is empty, then an empty regular expression object
+  /// pointer is returned.
+  ///
+  /// @return a pointer to the regular expression object of
+  /// variable_suppression::priv::symbol_name_not_regex_str_.
+  const sptr_utils::regex_t_sptr
+  get_symbol_name_not_regex() const
+  {
+    if (!symbol_name_not_regex_ && !symbol_name_not_regex_str_.empty())
+      {
+	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
+	if (regcomp(r.get(), symbol_name_not_regex_str_.c_str(),
+		    REG_EXTENDED == 0) == 0)
+	  symbol_name_not_regex_ = r;
+      }
+    return symbol_name_not_regex_;
+  }
+
   /// Getter for a pointer to a regular expression object built from
   /// the regular expression string
   /// variable_suppression::priv::symbol_version_regex_str_.
@@ -657,10 +748,12 @@ variable_is_suppressed(const ReadContextType&	ctxt,
       {
 	if (require_drop_property && !(*i)->get_drops_artifact_from_ir())
 	  continue;
-	if (ctxt.suppression_matches_variable_name(*suppr, var_name))
+	if (!var_name.empty()
+	    && ctxt.suppression_matches_variable_name(*suppr, var_name))
 	  return true;
-	if (ctxt.suppression_matches_variable_sym_name(*suppr,
-						       var_linkage_name))
+	if (!var_linkage_name.empty()
+	    && ctxt.suppression_matches_variable_sym_name(*suppr,
+							  var_linkage_name))
 	  return true;
       }
   return false;
@@ -675,6 +768,8 @@ class type_suppression::priv
   string				type_name_regex_str_;
   mutable sptr_utils::regex_t_sptr	type_name_regex_;
   string				type_name_;
+  string				type_name_not_regex_str_;
+  mutable sptr_utils::regex_t_sptr	type_name_not_regex_;
   bool					consider_type_kind_;
   type_suppression::type_kind		type_kind_;
   bool					consider_reach_kind_;
@@ -734,6 +829,56 @@ public:
   set_type_name_regex(sptr_utils::regex_t_sptr r)
   {type_name_regex_ = r;}
 
+  /// Get the regular expression object associated to the
+  /// 'type_name_not_regex' property of @ref type_suppression.
+  ///
+  /// If the regular expression object is not created, this method
+  /// creates it and returns it.
+  ///
+  /// If the 'type_name_not_regex' property of @ref type_suppression is
+  /// empty then this method returns nil.
+  const sptr_utils::regex_t_sptr
+  get_type_name_not_regex() const
+  {
+    if (!type_name_not_regex_)
+      {
+	if (!type_name_not_regex_str_.empty())
+	  {
+	    sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
+	    if (regcomp(r.get(),
+			type_name_not_regex_str_.c_str(),
+			REG_EXTENDED) == 0)
+	      type_name_not_regex_ = r;
+	  }
+      }
+    return type_name_not_regex_;
+  }
+
+  /// Setter for the type_name_not_regex object.
+  ///
+  /// @param r the new type_name_not_regex object.
+  void
+  set_type_name_not_regex(sptr_utils::regex_t_sptr r)
+  {type_name_not_regex_ = r;}
+
+  /// Getter for the string that denotes the 'type_name_not_regex'
+  /// property.
+  ///
+  /// @return the value of the string value of the
+  /// 'type_name_not_regex' property.
+  const string&
+  get_type_name_not_regex_str() const
+  {return type_name_not_regex_str_;}
+
+  /// Setter for the string that denotes the 'type_name_not_regex'
+  /// property.
+  ///
+  /// @return the value of the string value of the
+  /// 'type_name_not_regex' property.
+  void
+  set_type_name_not_regex_str(const string regex_str)
+  {type_name_not_regex_str_ = regex_str;}
+
   /// Getter for the source_location_to_keep_regex object.
   ///
   /// This function builds the regex if it's not yet built.
@@ -807,8 +952,7 @@ template <typename ReadContextType>
 bool
 type_is_suppressed(const ReadContextType&	ctxt,
 		   const string&		type_name,
-		   const location&		type_location,
-		   bool require_drop_property = false)
+		   const location&		type_location)
 {
   bool type_is_private = false;
   return type_is_suppressed(ctxt, type_name, type_location, type_is_private);
@@ -864,6 +1008,33 @@ type_is_suppressed(const ReadContextType&	ctxt,
   return false;
 }
 
+/// Test if a given ELF symbol is suppressed by a suppression
+/// specification.
+///
+/// @param ctxt the read context to use.
+///
+/// @param sym_name the name of the symbol to consider.
+///
+/// @param sym_type the type of the symbol to consider.
+///
+/// @return true iff the elf symbol denoted by @p sym_name and @p
+/// sym_type is suppressed.
+template<typename ReadContextType>
+bool
+is_elf_symbol_suppressed(const ReadContextType& ctxt,
+			 const string& sym_name,
+			 elf_symbol::type sym_type)
+{
+  if (elf_symbol_is_function(sym_type))
+    return suppr::function_is_suppressed(ctxt, /*fn_name=*/"",
+					 /*symbol_name=*/sym_name);
+  else if (elf_symbol_is_variable(sym_type))
+    return suppr::variable_is_suppressed(ctxt, /*var_name=*/"",
+					 /*symbol_name=*/sym_name);
+
+  return false;
+}
+
 // </type_suppression stuff>
 
 }// end namespace suppr
