@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -32,30 +32,27 @@
 // For package configuration macros.
 #include "config.h"
 
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <dirent.h>
-#include <time.h>
-#include <cstdlib>
-#include <cstring>
-#include <ctype.h>
-#include <errno.h>
-#include <libgen.h>
-#include <ext/stdio_filebuf.h> // For __gnu_cxx::stdio_filebuf
-// If fts.h is included before config.h, its indirect inclusions may
-// not give us the right LFS aliases of these functions, so map them
-// manually.
-#ifdef BAD_FTS
-  #ifdef _FILE_OFFSET_BITS
-    #define open open64
-    #define fopen fopen64
-  #endif
-#else
+// In case we don't have a bad fts then we need to include fts.h after
+// config.h.
+#ifndef BAD_FTS
   #include <sys/types.h>
   #include <fts.h>
 #endif
 
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <dirent.h>
+#include <time.h>
+#include <algorithm>
+#include <cstdlib>
+#include <cstring>
+#include <iterator>
+#include <ctype.h>
+#include <errno.h>
+#include <libgen.h>
+
 #include <fstream>
 #include <iostream>
 #include <sstream>
@@ -63,6 +60,7 @@
 
 #include "abg-dwarf-reader.h"
 #include "abg-internal.h"
+#include "abg-cxx-compat.h"
 // <headers defining libabigail's API go under here>
 ABG_BEGIN_EXPORT_DECLARATIONS
 
@@ -187,6 +185,154 @@ abidiff_status_has_incompatible_abi_change(abidiff_status s)
   struct stat st; \
   memset(&st, 0, sizeof(st))
 
+// <class timer stuff>
+
+/// The private data type of the @ref timer class.
+struct timer::priv
+{
+  timer::kind timer_kind;
+  struct timeval begin_timeval;
+  struct timeval end_timeval;
+
+  priv(timer::kind k)
+    : timer_kind(k),
+      begin_timeval(),
+      end_timeval()
+  {}
+}; // end struct timer::priv
+
+/// Constructor of the @ref timer type.
+///
+/// @param k the kind of timer to instantiate.
+timer::timer(timer::kind k)
+  : priv_(new timer::priv(k))
+{
+  if (priv_->timer_kind == START_ON_INSTANTIATION_TIMER_KIND)
+    start();
+}
+
+/// Start the timer.
+///
+/// To stop the timer (and record the time elapsed since the timer was
+/// started), call the timer::stop member function.
+///
+/// @return true upon successful completion.
+bool
+timer::start()
+{
+  if (gettimeofday(&priv_->begin_timeval, 0))
+    return false;
+  return true;
+}
+
+/// Stop the timer.
+///
+/// This records the time elapsed since the timer was started using
+/// the timer::start member function.
+///
+/// @return true upon successful completion.
+bool
+timer::stop()
+{
+  if (gettimeofday(&priv_->end_timeval, 0))
+    return false;
+  return true;
+}
+
+/// Get the elapsed time in seconds.
+///
+/// @return the time elapsed between the invocation of the methods
+/// timer::start() and timer::stop, in seconds.
+time_t
+timer::value_in_seconds() const
+{return priv_->end_timeval.tv_sec - priv_->begin_timeval.tv_sec;}
+
+/// Get the elapsed time in hour:minutes:seconds:milliseconds.
+///
+/// @param hours out parameter. This is set to the number of hours elapsed.
+///
+/// @param minutes out parameter.  This is set to the number of minutes
+/// (passed the number of hours) elapsed.
+///
+/// @param seconds out parameter.  This is se to the number of
+/// seconds (passed the number of hours and minutes) elapsed.
+///
+/// @param milliseconds.  This is set ot the number of milliseconds
+/// (passed the number of hours, minutes and seconds) elapsed.
+///
+/// @return true upon successful completion.
+bool
+timer::value(time_t&		hours,
+	     time_t&		minutes,
+	     time_t&		seconds,
+	     time_t&	milliseconds) const
+{
+  time_t elapsed_seconds =
+    priv_->end_timeval.tv_sec - priv_->begin_timeval.tv_sec;
+  suseconds_t elapsed_usecs =
+    ((priv_->end_timeval.tv_sec * 1000000) + priv_->end_timeval.tv_usec)
+    - ((priv_->begin_timeval.tv_sec * 1000000) + priv_->begin_timeval.tv_usec);
+
+  milliseconds = 0;
+
+  hours = elapsed_seconds / 3600;
+  minutes = (elapsed_seconds % 3600) / 60;
+  seconds = (elapsed_seconds % 3600) % 60;
+  if (elapsed_seconds == 0)
+    milliseconds = elapsed_usecs / 1000;
+
+  return true;
+}
+
+/// Get the elapsed time as a human-readable string.
+///
+/// @return the elapsed time as a human-readable string.
+string
+timer::value_as_string() const
+{
+  time_t hours = 0, minutes = 0, seconds = 0;
+  time_t msecs = 0;
+
+  value(hours, minutes, seconds, msecs);
+
+  std::ostringstream o;
+
+  if (hours)
+    o << hours << "h";
+
+  if (minutes)
+    o << minutes << "m";
+
+  o << seconds << "s";
+
+  if (msecs)
+    o <<msecs <<"ms";
+
+  return o.str();
+}
+
+/// Destructor of the @ref timer type.
+timer::~timer()
+{
+}
+
+/// Streaming operator for the @ref timer type.
+///
+/// Emit a string representing the elapsed time (in a human-readable
+/// manner) to an output stream.
+///
+/// @param o the output stream to emit the elapsed time string to.
+///
+/// @param t the timer to consider.
+///
+/// @return the output stream considered.
+ostream&
+operator<<(ostream& o, const timer& t)
+{
+  o << t.value_as_string();
+  return o;
+}
+
 /// Get the stat struct (as returned by the lstat() function of the C
 /// library) of a file.  Note that the function uses lstat, so that
 /// callers can detect symbolic links.
@@ -294,6 +440,131 @@ is_dir(const string& path)
   return false;
 }
 
+static const char* ANONYMOUS_STRUCT_INTERNAL_NAME = "__anonymous_struct__";
+static const char* ANONYMOUS_UNION_INTERNAL_NAME =  "__anonymous_union__";
+static const char* ANONYMOUS_ENUM_INTERNAL_NAME =   "__anonymous_enum__";
+
+static int ANONYMOUS_STRUCT_INTERNAL_NAME_LEN =
+  strlen(ANONYMOUS_STRUCT_INTERNAL_NAME);
+
+static int ANONYMOUS_UNION_INTERNAL_NAME_LEN =
+  strlen(ANONYMOUS_UNION_INTERNAL_NAME);
+
+static int ANONYMOUS_ENUM_INTERNAL_NAME_LEN =
+  strlen(ANONYMOUS_ENUM_INTERNAL_NAME);
+
+/// Getter of the prefix for the name of anonymous structs.
+///
+/// @reaturn the prefix for the name of anonymous structs.
+const char*
+get_anonymous_struct_internal_name_prefix()
+{return ANONYMOUS_STRUCT_INTERNAL_NAME;}
+
+/// Getter of the prefix for the name of anonymous unions.
+///
+/// @reaturn the prefix for the name of anonymous unions.
+const char*
+get_anonymous_union_internal_name_prefix()
+{return ANONYMOUS_UNION_INTERNAL_NAME;}
+
+/// Getter of the prefix for the name of anonymous enums.
+///
+/// @reaturn the prefix for the name of anonymous enums.
+const char*
+get_anonymous_enum_internal_name_prefix()
+{return ANONYMOUS_ENUM_INTERNAL_NAME;}
+
+/// Compare two fully qualified decl names by taking into account that
+/// they might have compontents that are anonymous types/namespace names.
+///
+/// For instance:
+///
+/// __anonymous_struct__1::foo and __anonymous_struct__2::foo are
+/// considered being equivalent qualified names because both are data
+/// members that belong to anonymous structs.  The anonymous structs
+/// are numbered so that we can tell them appart (and look them up)
+/// where there are several of them in the same scope.  But during
+/// comparison, for various purposes, we want to consider them as
+/// equivalent.
+///
+/// Similarly, __anonymous_struct__1::foo::__anonymous_struct__2::bar
+/// and __anonymous_struct__10::foo::__anonymous_struct__11::bar are
+/// equivalent.
+///
+/// But __anonymous_struct__1::foo::__anonymous_struct__2::bar and
+/// __anonymous_struct__10::foo::__anonymous_union__11::bar are not
+/// equivalent because the former designates a member of an anonymous
+/// struct and the latter designates a member of an anonymous union.
+///
+/// So this function handles those cases.
+///
+/// @param l the name of the first (left hand side) decl to consider.
+///
+/// @param r the name of the second (right hand side) decl to consider.
+///
+/// @return true iff @p l is equivalent to @p r when taking into
+/// account the anonymous scopes that both might have and if they
+/// might be anonymous themselves.
+bool
+decl_names_equal(const string& l, const string& r)
+{
+  string::size_type l_pos1 = 0, l_pos2 = 0, r_pos1 = 0, r_pos2 = 0;
+  string::size_type l_length = l.length(), r_length = r.length();
+
+  while (l_pos1 < l_length && r_pos1 < r_length)
+    {
+      l_pos2 = l.find("::", l_pos1);
+      r_pos2 = r.find("::", r_pos1);
+
+      if ((l.compare(l_pos1,
+		     ANONYMOUS_STRUCT_INTERNAL_NAME_LEN,
+		     ANONYMOUS_STRUCT_INTERNAL_NAME) == 0
+	   && r.compare(r_pos1,
+			ANONYMOUS_STRUCT_INTERNAL_NAME_LEN,
+			ANONYMOUS_STRUCT_INTERNAL_NAME) == 0)
+	  ||
+	  (l.compare(l_pos1,
+		     ANONYMOUS_UNION_INTERNAL_NAME_LEN,
+		     ANONYMOUS_UNION_INTERNAL_NAME) == 0
+	   && r.compare(r_pos1,
+			ANONYMOUS_UNION_INTERNAL_NAME_LEN,
+			ANONYMOUS_UNION_INTERNAL_NAME) == 0)
+	  ||
+	  (l.compare(l_pos1,
+		     ANONYMOUS_ENUM_INTERNAL_NAME_LEN,
+		     ANONYMOUS_ENUM_INTERNAL_NAME) == 0
+	   && r.compare(r_pos1,
+			ANONYMOUS_ENUM_INTERNAL_NAME_LEN,
+			ANONYMOUS_ENUM_INTERNAL_NAME) == 0))
+	{
+	  if (l_pos2 == l.npos || r_pos2 == r.npos)
+	    return true;
+
+	  l_pos1 = l_pos2 + 2;
+	  r_pos1 = r_pos2 + 2;
+	  continue;
+	}
+
+      if (l_pos2 == l.npos || r_pos2 == r.npos)
+	{
+	  if (l_pos2 != r_pos2)
+	    return false;
+
+	  return !l.compare(l_pos1, l_pos2, r,
+			    r_pos1, r_pos2);
+	}
+
+      if (l.compare(l_pos1, l_pos2 - l_pos1, r,
+		    r_pos1, r_pos2 - r_pos1))
+	return false;
+
+      l_pos1 = l_pos2 + 2;
+      r_pos1 = r_pos2 + 2;
+    }
+
+  return true;
+}
+
 /// If a given file is a symbolic link, get the canonicalized absolute
 /// path to the target file.
 ///
@@ -405,7 +676,10 @@ real_path(const string&path, string& result)
 
   char *realp = realpath(path.c_str(), NULL);
   if (realp)
-    result = realp;
+    {
+      result = realp;
+      free(realp);
+    }
 }
 
 /// Ensures #dir_path is a directory and is created.  If #dir_path is
@@ -787,9 +1061,9 @@ sorted_strings_common_prefix(vector<string>& input_strings, string& prefix)
 string
 get_library_version_string()
 {
-  string major, minor, revision, version_string;
-  abigail::abigail_get_library_version(major, minor, revision);
-  version_string = major + "." + minor + "." + revision;
+  string major, minor, revision, version_string, suffix;
+  abigail::abigail_get_library_version(major, minor, revision, suffix);
+  version_string = major + "." + minor + "." + revision + suffix;
   return version_string;
 }
 
@@ -943,10 +1217,9 @@ convert_char_stars_to_char_star_stars(const vector<char*> &char_stars,
 /// The private data of the @ref temp_file type.
 struct temp_file::priv
 {
-  char*					path_template_;
-  int						fd_;
-  shared_ptr<__gnu_cxx::stdio_filebuf<char> >	filebuf_;
-  shared_ptr<std::iostream>			iostream_;
+  char*			   path_template_;
+  int			   fd_;
+  shared_ptr<std::fstream> fstream_;
 
   priv()
   {
@@ -960,18 +1233,17 @@ struct temp_file::priv
     if (fd_ == -1)
       return;
 
-    using __gnu_cxx::stdio_filebuf;
-    filebuf_.reset(new stdio_filebuf<char>(fd_,
-					   std::ios::in | std::ios::out));
-    iostream_.reset(new std::iostream(filebuf_.get()));
+    fstream_.reset(new std::fstream(path_template_,
+				    std::ios::trunc
+				    | std::ios::in
+				    | std::ios::out));
   }
 
   ~priv()
   {
     if (fd_ && fd_ != -1)
       {
-	iostream_.reset();
-	filebuf_.reset();
+	fstream_.reset();
 	close(fd_);
 	remove(path_template_);
       }
@@ -992,7 +1264,7 @@ temp_file::temp_file()
 /// useable.
 bool
 temp_file::is_good() const
-{return (priv_->fd_ && priv_->fd_ != -1);}
+{return priv_->fstream_->good();}
 
 /// Return the path to the temporary file.
 ///
@@ -1007,19 +1279,19 @@ temp_file::get_path() const
   return 0;
 }
 
-/// Get the iostream to the temporary file.
+/// Get the fstream to the temporary file.
 ///
 /// Note that the current process is aborted if this member function
 /// is invoked on an instance of @ref temp_file that is not usable.
 /// So please test that the instance is usable by invoking the
 /// temp_file::is_good() member function on it first.
 ///
-/// @return the iostream to the temporary file.
-std::iostream&
+/// @return the fstream to the temporary file.
+std::fstream&
 temp_file::get_stream()
 {
   ABG_ASSERT(is_good());
-  return *priv_->iostream_;
+  return *priv_->fstream_;
 }
 
 /// Create the temporary file and return it if it's usable.
@@ -1466,10 +1738,10 @@ struct malloced_char_star_deleter
 /// @param p the path to turn into an absolute path.
 ///
 /// @return a shared pointer to the resulting absolute path.
-std::tr1::shared_ptr<char>
+abg_compat::shared_ptr<char>
 make_path_absolute(const char*p)
 {
-  using std::tr1::shared_ptr;
+  using abg_compat::shared_ptr;
 
   shared_ptr<char> result;
 
@@ -1607,87 +1879,108 @@ gen_suppr_spec_from_headers(const string& headers_root_dir)
 /// names, one name per line.  Each function or variable name is the
 /// name of a function or a variable whose changes are to be keept.
 ///
-/// This function reads the white list and generates a
-/// function_suppression_sptr or variable_suppression_sptr (or
-/// several, if there are more than one section) that is added to a
-/// vector of suppressions.
+/// A whitelist file can have multiple sections (adhering to the naming
+/// conventions and multiple files can be passed. The suppression that
+/// is created takes all whitelist sections from all files into account.
+/// Symbols (or expression of such) are deduplicated in the final
+/// suppression expression.
 ///
-/// @param abi_whitelist_path the path to the Kernel ABI whitelist.
+/// This function reads the white lists and generates a
+/// function_suppression_sptr and variable_suppression_sptr and returns
+/// a vector containing those.
 ///
-/// @param supprs the resulting vector of suppressions to which the
-/// new function suppressions resulting from reading the whitelist are
-/// added.  This vector is updated iff the function returns true.
+/// @param abi_whitelist_paths a vector of KMI whitelist paths
 ///
-/// @return true iff the abi whitelist file was read and function
-/// suppressions could be generated as a result.
-bool
-gen_suppr_spec_from_kernel_abi_whitelist(const string& abi_whitelist_path,
-					 suppressions_type& supprs)
+/// @return a vector or suppressions
+suppressions_type
+gen_suppr_spec_from_kernel_abi_whitelists
+   (const std::vector<std::string>& abi_whitelist_paths)
 {
-  abigail::ini::config whitelist;
-  if (!read_config(abi_whitelist_path, whitelist))
-    return false;
 
-  bool created_a_suppr = false;
-
-  const ini::config::sections_type &whitelist_sections =
-    whitelist.get_sections();
-  for (ini::config::sections_type::const_iterator s =
-	 whitelist_sections.begin();
-       s != whitelist_sections.end();
-       ++s)
+  std::vector<std::string> whitelisted_names;
+  for (std::vector<std::string>::const_iterator
+	   path_iter = abi_whitelist_paths.begin(),
+	   path_end = abi_whitelist_paths.end();
+       path_iter != path_end;
+       ++path_iter)
     {
-      string section_name = (*s)->get_name();
-      if (!string_ends_with(section_name, "whitelist"))
+
+      abigail::ini::config whitelist;
+      if (!read_config(*path_iter, whitelist))
 	continue;
 
-      function_suppression_sptr fn_suppr;
-      variable_suppression_sptr var_suppr;
-      string function_names_regexp, variable_names_regexp;
-      for (ini::config::properties_type::const_iterator p =
-	     (*s)->get_properties().begin();
-	   p != (*s)->get_properties().end();
-	   ++p)
+      const ini::config::sections_type& whitelist_sections =
+	  whitelist.get_sections();
+
+      for (ini::config::sections_type::const_iterator
+	       section_iter = whitelist_sections.begin(),
+	       section_end = whitelist_sections.end();
+	   section_iter != section_end;
+	   ++section_iter)
 	{
-	  if (simple_property_sptr prop = is_simple_property(*p))
-	    if (prop->has_empty_value())
-	      {
-		const string &name = prop->get_name();
-		if (!name.empty())
+	  std::string section_name = (*section_iter)->get_name();
+	  if (!string_ends_with(section_name, "whitelist"))
+	    continue;
+	  for (ini::config::properties_type::const_iterator
+		   prop_iter = (*section_iter)->get_properties().begin(),
+		   prop_end = (*section_iter)->get_properties().end();
+	       prop_iter != prop_end;
+	       ++prop_iter)
+	    {
+	      if (const simple_property_sptr& prop =
+		      is_simple_property(*prop_iter))
+		if (prop->has_empty_value())
 		  {
-		    if (!function_names_regexp.empty())
-		      function_names_regexp += "|";
-		    function_names_regexp += "^" + name + "$";
-
-		    if (!variable_names_regexp.empty())
-		      variable_names_regexp += "|";
-		    variable_names_regexp += "^" + name + "$";
+		    const std::string& name = prop->get_name();
+		    if (!name.empty())
+		      whitelisted_names.push_back(name);
 		  }
-	      }
-	}
-
-      if (!function_names_regexp.empty())
-	{
-	  fn_suppr.reset(new function_suppression);
-	  fn_suppr->set_label(section_name);
-	  fn_suppr->set_name_not_regex_str(function_names_regexp);
-	  fn_suppr->set_drops_artifact_from_ir(true);
-	  supprs.push_back(fn_suppr);
-	  created_a_suppr = true;
-	}
-
-      if (!variable_names_regexp.empty())
-	{
-	  var_suppr.reset(new variable_suppression);
-	  var_suppr->set_label(section_name);
-	  var_suppr->set_name_not_regex_str(variable_names_regexp);
-	  var_suppr->set_drops_artifact_from_ir(true);
-	  supprs.push_back(var_suppr);
-	  created_a_suppr = true;
+	    }
 	}
     }
 
-  return created_a_suppr;
+  suppressions_type result;
+  if (!whitelisted_names.empty())
+    {
+      // Drop duplicates to simplify the regex we are generating
+      std::sort(whitelisted_names.begin(), whitelisted_names.end());
+      whitelisted_names.erase(std::unique(whitelisted_names.begin(),
+					  whitelisted_names.end()),
+			      whitelisted_names.end());
+
+      // Build a regular expression representing the union of all
+      // the function and variable names expressed in the white list.
+      std::stringstream regex_ss;
+      regex_ss << "^";
+      std::copy(whitelisted_names.begin(), whitelisted_names.end(),
+		std::ostream_iterator<std::string>(regex_ss, "$|^"));
+      regex_ss.seekp(0, std::ios::end);
+      const std::string& regex =
+	  regex_ss.str().substr(0, static_cast<size_t>(regex_ss.tellp()) - 2);
+
+      // Build a suppression specification which *keeps* functions
+      // whose ELF symbols match the regular expression contained
+      // in function_names_regexp.  This will also keep the ELF
+      // symbols (not designated by any debug info) whose names
+      // match this regexp.
+      function_suppression_sptr fn_suppr(new function_suppression);
+      fn_suppr->set_label("whitelist");
+      fn_suppr->set_symbol_name_not_regex_str(regex);
+      fn_suppr->set_drops_artifact_from_ir(true);
+      result.push_back(fn_suppr);
+
+      // Build a suppression specification which *keeps* variables
+      // whose ELF symbols match the regular expression contained
+      // in function_names_regexp.  This will also keep the ELF
+      // symbols (not designated by any debug info) whose names
+      // match this regexp.
+      variable_suppression_sptr var_suppr(new variable_suppression);
+      var_suppr->set_label("whitelist");
+      var_suppr->set_symbol_name_not_regex_str(regex);
+      var_suppr->set_drops_artifact_from_ir(true);
+      result.push_back(var_suppr);
+    }
+  return result;
 }
 
 /// Get the path to the default system suppression file.
@@ -1828,6 +2121,8 @@ find_file_under_dir(const string& root_dir,
 	  return true;
 	}
     }
+
+  fts_close(file_hierarchy);
   return false;
 }
 /// If we were given suppression specification files or kabi whitelist
@@ -1862,11 +2157,10 @@ load_generate_apply_suppressions(dwarf_reader::read_context &read_ctxt,
 	   ++i)
 	read_suppressions(*i, supprs);
 
-      for (vector<string>::const_iterator i =
-	     kabi_whitelist_paths.begin();
-	   i != kabi_whitelist_paths.end();
-	   ++i)
-	gen_suppr_spec_from_kernel_abi_whitelist(*i, supprs);
+      const suppressions_type& wl_suppr =
+	  gen_suppr_spec_from_kernel_abi_whitelists(kabi_whitelist_paths);
+
+      supprs.insert(supprs.end(), wl_suppr.begin(), wl_suppr.end());
     }
 
   abigail::dwarf_reader::add_read_context_suppressions(read_ctxt, supprs);
@@ -2027,7 +2321,8 @@ find_vmlinux_path(const string&	from,
 /// binary that was found.
 ///
 /// @param module_paths output parameter.  The paths of the kernel
-/// module binaries that were found.
+/// module binaries that were found, sorted to impose a deterministic
+/// ordering.
 ///
 /// @return true if at least the path to the vmlinux binary was found.
 bool
@@ -2066,6 +2361,8 @@ get_binary_paths_from_kernel_dist(const string&	dist_root,
   if (find_vmlinux_and_module_paths(from, vmlinux_path, module_paths))
     found = true;
 
+  std::sort(module_paths.begin(), module_paths.end());
+
   return found;
 }
 
@@ -2177,14 +2474,18 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
   vector<string> modules;
 
   if (verbose)
-    std::cout << "Analysing kernel dist root '"
+    std::cerr << "Analysing kernel dist root '"
 	      << root << "' ... " << std::flush;
 
+  timer t;
+
+  t.start();
   bool got_binary_paths =
     get_binary_paths_from_kernel_dist(root, debug_info_root, vmlinux, modules);
+  t.stop();
 
   if (verbose)
-    std::cout << "DONE\n";
+    std::cerr << "DONE: " << t << "\n";
 
   dwarf_reader::read_context_sptr ctxt;
   if (got_binary_paths)
@@ -2202,9 +2503,17 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
 	    dwarf_reader::create_read_context(vmlinux, di_roots ,env.get(),
 					      /*read_all_types=*/false,
 					      /*linux_kernel_mode=*/true);
+	  dwarf_reader::set_do_log(*ctxt, verbose);
 
+	  t.start();
 	  load_generate_apply_suppressions(*ctxt, suppr_paths,
 					   kabi_wl_paths, supprs);
+	  t.stop();
+
+	  if (verbose)
+	    std::cerr << "loaded white list and generated suppr spec in: "
+		      << t
+		      << "\n";
 
 	  // If we have been given a whitelist of functions and
 	  // variable symbols to look at, then we can avoid loading
@@ -2217,14 +2526,18 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
 	  set_read_context_corpus_group(*ctxt, group);
 
 	  if (verbose)
-	    std::cout << "reading kernel binary '"
-		      << vmlinux << "' ... " << std::flush;
+	    std::cerr << "reading kernel binary '"
+		      << vmlinux << "' ...\n" << std::flush;
 
 	  // Read the vmlinux corpus and add it to the group.
+	  t.start();
 	  read_and_add_corpus_to_group_from_elf(*ctxt, *group, status);
+	  t.stop();
 
-	    if (verbose)
-	      std::cout << " DONE\n";
+	  if (verbose)
+	    std::cerr << vmlinux
+		      << " reading DONE:"
+		      << t << "\n";
 	}
 
       if (!group->is_empty())
@@ -2237,7 +2550,7 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
 	       ++m, ++cur_module_index)
 	    {
 	      if (verbose)
-		std::cout << "reading module '"
+		std::cerr << "reading module '"
 			  << *m << "' ("
 			  << cur_module_index
 			  << "/" << total_nb_modules
@@ -2259,10 +2572,15 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
 
 	      set_read_context_corpus_group(*ctxt, group);
 
+	      t.start();
 	      read_and_add_corpus_to_group_from_elf(*ctxt,
 						    *group, status);
+	      t.stop();
 	      if (verbose)
-		std::cout << " DONE\n";
+		std::cerr << "module '"
+			  << *m
+			  << "' reading DONE: "
+			  << t << "\n";
 	    }
 
 	  result = group;
