@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -542,6 +542,34 @@ InOutSpec in_out_specs[] =
     "data/test-diff-filter/test47-filter-void-ptr-change-report-0.txt",
     "output/test-diff-filter/test47-filter-void-ptr-change-report-0.txt",
   },
+  {
+    "data/test-diff-filter/PR24430-fold-qualified-array-clang",
+    "data/test-diff-filter/PR24430-fold-qualified-array-gcc",
+    "--no-default-suppression",
+    "data/test-diff-filter/PR24430-fold-qualified-array-report-0.txt",
+    "output/test-diff-filter/PR24430-fold-qualified-array-report-0.txt",
+  },
+  {
+    "data/test-diff-filter/test-PR24731-v0.o ",
+    "data/test-diff-filter/test-PR24731-v1.o ",
+    "--no-default-suppression",
+    "data/test-diff-filter/test-PR24731-report-0.txt",
+    "output/test-diff-filter/test-PR24731-report-0.txt",
+  },
+  {
+    "data/test-diff-filter/test-PR24731-v0.o ",
+    "data/test-diff-filter/test-PR24731-v1.o ",
+    "--no-default-suppression --harmless",
+    "data/test-diff-filter/test-PR24731-report-1.txt",
+    "output/test-diff-filter/test-PR24731-report-1.txt",
+  },
+  {
+    "data/test-diff-filter/PR24787-libone.so",
+    "data/test-diff-filter/PR24787-libtwo.so",
+    "--no-default-suppression",
+    "data/test-diff-filter/PR24787-report-0.txt",
+    "output/test-diff-filter/PR24787-report-0.txt",
+  },
   // This should be the last entry
   {NULL, NULL, NULL, NULL, NULL}
 };
@@ -639,7 +667,7 @@ int
 main()
 {
   using std::vector;
-  using std::tr1::dynamic_pointer_cast;
+  using abg_compat::dynamic_pointer_cast;
   using abigail::workers::queue;
   using abigail::workers::task;
   using abigail::workers::task_sptr;
