@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -1748,6 +1748,266 @@ InOutSpec in_out_specs[] =
     "data/test-diff-suppr/test41-enumerator-changes-report-0.txt",
     "output/test-diff-suppr/test41-enumerator-changes-report-0.txt"
   },
+  {
+    "data/test-diff-suppr/test42-negative-suppr-type-v0.o",
+    "data/test-diff-suppr/test42-negative-suppr-type-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test42-negative-suppr-type-suppr-1.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/test42-negative-suppr-type-report-0.txt",
+    "output/test-diff-suppr/test42-negative-suppr-type-report-0.txt"
+  },
+  {
+    "data/test-diff-suppr/test42-negative-suppr-type-v0.o",
+    "data/test-diff-suppr/test42-negative-suppr-type-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test42-negative-suppr-type-suppr-2.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/test42-negative-suppr-type-report-1.txt",
+    "output/test-diff-suppr/test42-negative-suppr-type-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/test43-suppr-direct-fn-subtype-v0.o",
+    "data/test-diff-suppr/test43-suppr-direct-fn-subtype-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test43-suppr-direct-fn-subtype-suppr-1.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/test43-suppr-direct-fn-subtype-report-1.txt",
+    "output/test-diff-suppr/test43-suppr-direct-fn-subtype-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-v0.o",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-v1.o",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-report-1.txt",
+    "output/test-diff-suppr/test44-suppr-sym-name-not-regexp-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-v0.o",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp.suppr.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-report-2.txt",
+    "output/test-diff-suppr/test44-suppr-sym-name-not-regexp-report-2.txt"
+  },
+  {
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-v0.o.abi",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-v1.o.abi",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-report-1.txt",
+    "output/test-diff-suppr/test44-suppr-sym-name-not-regexp-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-v0.o.abi",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-v1.o.abi",
+    "",
+    "",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp.suppr.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/test44-suppr-sym-name-not-regexp-report-2.txt",
+    "output/test-diff-suppr/test44-suppr-sym-name-not-regexp-report-2.txt"
+  },
+  {
+    "data/test-diff-suppr/test45-abi.xml",
+    "data/test-diff-suppr/test45-abi-wl.xml",
+    "",
+    "",
+    "data/test-diff-suppr/test45-abi.suppr.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/test45-abi-report-1.txt",
+    "output/test-diff-suppr/test45-abi-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/test46-PR25128-base.xml",
+    "data/test-diff-suppr/test46-PR25128-new.xml",
+    "",
+    "",
+    "",
+    "--no-default-suppression --leaf-changes-only",
+    "data/test-diff-suppr/test46-PR25128-report-1.txt",
+    "output/test-diff-suppr/test46-PR25128-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/test46-PR25128-base.xml",
+    "data/test-diff-suppr/test46-PR25128-new.xml",
+    "",
+    "",
+    "",
+    "--no-default-suppression --leaf-changes-only --no-added-syms",
+    "data/test-diff-suppr/test46-PR25128-report-2.txt",
+    "output/test-diff-suppr/test46-PR25128-report-2.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o",
+    "",
+    "",
+    "",
+    "--no-default-suppression --non-reachable-types",
+    "data/test-diff-suppr/test47-non-reachable-types-report-1.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test47-non-reachable-types-suppr-1.txt",
+    "--no-default-suppression --non-reachable-types",
+    "data/test-diff-suppr/test47-non-reachable-types-report-2.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-2.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test47-non-reachable-types-suppr-2.txt",
+    "--no-default-suppression --non-reachable-types",
+    "data/test-diff-suppr/test47-non-reachable-types-report-3.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-3.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test47-non-reachable-types-suppr-3.txt",
+    "--no-default-suppression --non-reachable-types",
+    "data/test-diff-suppr/test47-non-reachable-types-report-4.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-4.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test47-non-reachable-types-suppr-4.txt",
+    "--no-default-suppression --non-reachable-types",
+    "data/test-diff-suppr/test47-non-reachable-types-report-5.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-5.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test47-non-reachable-types-suppr-5.txt",
+    "--no-default-suppression --non-reachable-types",
+    "data/test-diff-suppr/test47-non-reachable-types-report-6.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-6.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o",
+    "",
+    "",
+    "",
+    "--no-default-suppression --non-reachable-types --leaf-changes-only",
+    "data/test-diff-suppr/test47-non-reachable-types-report-7.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-7.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test47-non-reachable-types-suppr-4.txt",
+    "--no-default-suppression --non-reachable-types --leaf-changes-only",
+    "data/test-diff-suppr/test47-non-reachable-types-report-8.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-8.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o",
+    "",
+    "",
+    "data/test-diff-suppr/test47-non-reachable-types-suppr-5.txt",
+    "--no-default-suppression --non-reachable-types --leaf-changes-only",
+    "data/test-diff-suppr/test47-non-reachable-types-report-9.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-9.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o.alltypes.abixml",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o.alltypes.abixml",
+    "",
+    "",
+    "",
+    "--no-default-suppression --non-reachable-types",
+    "data/test-diff-suppr/test47-non-reachable-types-report-1.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/test47-non-reachable-types-v0.o.alltypes.abixml",
+    "data/test-diff-suppr/test47-non-reachable-types-v1.o.alltypes.abixml",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    "data/test-diff-suppr/test47-non-reachable-types-report-10.txt",
+    "output/test-diff-suppr/test47-non-reachable-types-report-10.txt"
+  },
+  {
+    "data/test-diff-suppr/libtest48-soname-abixml-v0.so",
+    "data/test-diff-suppr/libtest48-soname-abixml-v1.so.abi",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    "data/test-diff-suppr/libtest48-soname-abixml-report-1.txt",
+    "output/test-diff-suppr/libtest48-soname-abixml-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/libtest48-soname-abixml-v0.so",
+    "data/test-diff-suppr/libtest48-soname-abixml-v1.so.abi",
+    "",
+    "",
+    "data/test-diff-suppr/libtest48-soname-abixml-suppr.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/libtest48-soname-abixml-report-2.txt",
+    "output/test-diff-suppr/libtest48-soname-abixml-report-2.txt"
+  },
+  {
+    "data/test-diff-suppr/libtest48-soname-abixml-v0.so",
+    "data/test-diff-suppr/libtest48-soname-abixml-v1.so.abi",
+    "",
+    "",
+    "data/test-diff-suppr/libtest48-soname-abixml-suppr-2.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/libtest48-soname-abixml-report-1.txt",
+    "output/test-diff-suppr/libtest48-soname-abixml-report-1.txt"
+  },
+  {
+    "data/test-diff-suppr/libtest48-soname-abixml-v0.so",
+    "data/test-diff-suppr/libtest48-soname-abixml-v1.so.abi",
+    "",
+    "",
+    "data/test-diff-suppr/libtest48-soname-abixml-suppr-3.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/libtest48-soname-abixml-report-2.txt",
+    "output/test-diff-suppr/libtest48-soname-abixml-report-2.txt"
+  },
+  {
+    "data/test-diff-suppr/libtest48-soname-abixml-v0.so",
+    "data/test-diff-suppr/libtest48-soname-abixml-v1.so.abi",
+    "",
+    "",
+    "data/test-diff-suppr/libtest48-soname-abixml-suppr-4.txt",
+    "--no-default-suppression",
+    "data/test-diff-suppr/libtest48-soname-abixml-report-1.txt",
+    "output/test-diff-suppr/libtest48-soname-abixml-report-1.txt"
+  },
   // This should be the last entry
   {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL}
 };
