@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -40,12 +40,15 @@ using std::vector;
 using std::string;
 using std::ofstream;
 using std::cerr;
-using std::tr1::dynamic_pointer_cast;
+using abg_compat::dynamic_pointer_cast;
 using abigail::tests::get_build_dir;
 using abigail::dwarf_reader::read_corpus_from_elf;
 using abigail::dwarf_reader::read_context;
 using abigail::dwarf_reader::read_context_sptr;
 using abigail::dwarf_reader::create_read_context;
+using abigail::xml_writer::create_write_context;
+using abigail::xml_writer::write_context_sptr;
+using abigail::xml_writer::write_corpus;
 
 /// This is an aggregate that specifies where a test shall get its
 /// input from, and where it shall write its ouput to.
@@ -234,6 +237,34 @@ InOutSpec in_out_specs[] =
     "data/test-read-dwarf/PR24378-fn-is-not-scope.abi",
     "output/test-read-dwarf/PR24378-fn-is-not-scope.abi",
   },
+#if defined(HAVE_R_AARCH64_ABS64_MACRO) && defined(HAVE_R_AARCH64_PREL32_MACRO)
+  {
+    "data/test-read-dwarf/PR25007-sdhci.ko",
+    "",
+    "data/test-read-dwarf/PR25007-sdhci.ko.abi",
+    "output/test-read-dwarf/PR25007-sdhci.ko.abi",
+  },
+#endif
+#if defined HAVE_DW_FORM_strx
+  {
+    "data/test-read-dwarf/PR25042-libgdbm-clang-dwarf5.so.6.0.0",
+    "",
+    "data/test-read-dwarf/PR25042-libgdbm-clang-dwarf5.so.6.0.0.abi",
+    "output/test-read-dwarf/PR25042-libgdbm-clang-dwarf5.so.6.0.0.abi",
+  },
+#endif
+  {
+    "data/test-read-dwarf/test25-bogus-binary.elf",
+    "",
+    "",
+    "",
+  },
+  {
+    "data/test-read-dwarf/test26-bogus-binary.elf",
+    "",
+    "",
+    "",
+  },
   // This should be the last entry.
   {NULL, NULL, NULL, NULL}
 };
@@ -305,6 +336,12 @@ struct test_task : public abigail::workers::task
       set_suppressions(*ctxt, in_suppr_spec_path);
 
     abigail::corpus_sptr corp = read_corpus_from_elf(*ctxt, status);
+    // if there is no output and no input, assume that we do not care about the
+    // actual read result, just that it succeeded.
+    if (in_abi_path.empty() && out_abi_path.empty()) {
+	// Phew! we made it here and we did not crash! yay!
+	return;
+    }
     if (!corp)
       {
 	error_message = string("failed to read ") + in_elf_path  + "\n";
@@ -333,8 +370,9 @@ struct test_task : public abigail::workers::task
 	is_ok = false;
 	return;
       }
-    is_ok =
-      abigail::xml_writer::write_corpus(corp, /*indent=*/0, of);
+    const write_context_sptr write_ctxt
+	= create_write_context(corp->get_environment(), of);
+    is_ok = write_corpus(*write_ctxt, corp, /*indent=*/0);
     of.close();
 
     string abidw = string(get_build_dir()) + "/tools/abidw";
