@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -38,7 +38,7 @@ using std::string;
 using std::ostream;
 using std::cout;
 using std::cerr;
-using std::tr1::shared_ptr;
+using abg_compat::shared_ptr;
 using abigail::ir::environment;
 using abigail::ir::environment_sptr;
 using abigail::translation_unit;
@@ -59,7 +59,7 @@ using abigail::tools_utils::emit_prefix;
 using abigail::tools_utils::check_file;
 using abigail::tools_utils::guess_file_type;
 using abigail::tools_utils::gen_suppr_spec_from_headers;
-using abigail::tools_utils::gen_suppr_spec_from_kernel_abi_whitelist;
+using abigail::tools_utils::gen_suppr_spec_from_kernel_abi_whitelists;
 using abigail::tools_utils::load_default_system_suppressions;
 using abigail::tools_utils::load_default_user_suppressions;
 using abigail::tools_utils::abidiff_status;
@@ -101,6 +101,7 @@ struct options
   bool			show_changed_vars;
   bool			show_added_vars;
   bool			show_all_vars;
+  bool			show_all_types;
   bool			show_linkage_names;
   bool			show_locs;
   bool			show_harmful_changes;
@@ -141,6 +142,7 @@ struct options
       show_changed_vars(),
       show_added_vars(),
       show_all_vars(true),
+      show_all_types(false),
       show_linkage_names(true),
       show_locs(true),
       show_harmful_changes(true),
@@ -203,6 +205,8 @@ display_usage(const string& prog_name, ostream& out)
     << " --deleted-vars  display deleted global public variables\n"
     << " --changed-vars  display changed global public variables\n"
     << " --added-vars  display added global public variables\n"
+    << " --non-reachable-types|-t  consider types non reachable"
+    " from public interfaces\n"
     << " --no-added-syms  do not display added functions or variables\n"
     << " --no-linkage-name  do not display linkage names of "
     "added/removed/changed\n"
@@ -399,6 +403,9 @@ parse_command_line(int argc, char* argv[], options& opts)
 	  opts.show_all_fns = false;
 	  opts.show_all_vars = false;
 	}
+      else if (!strcmp(argv[i], "--non-reachable-types")
+	       || !strcmp(argv[i], "-t"))
+	  opts.show_all_types = true;
       else if (!strcmp(argv[i], "--no-added-syms"))
 	{
 	  opts.show_added_syms = false;
@@ -647,6 +654,7 @@ set_diff_context_from_opts(diff_context_sptr ctxt,
     (opts.show_symbols_not_referenced_by_debug_info);
   ctxt->show_added_symbols_unreferenced_by_debug_info
     (opts.show_symbols_not_referenced_by_debug_info && opts.show_added_syms);
+  ctxt->show_unreachable_types(opts.show_all_types);
   ctxt->show_impacted_interfaces(opts.show_impacted_interfaces);
 
   if (!opts.show_harmless_changes)
@@ -760,15 +768,29 @@ set_suppressions(ReadContextType& read_ctxt, const options& opts)
 	}
     }
 
-  for (vector<string>::const_iterator i =
-	 opts.kernel_abi_whitelist_paths.begin();
-       i != opts.kernel_abi_whitelist_paths.end();
-       ++i)
-    gen_suppr_spec_from_kernel_abi_whitelist(*i, supprs);
+  const suppressions_type& wl_suppr =
+      gen_suppr_spec_from_kernel_abi_whitelists(
+	  opts.kernel_abi_whitelist_paths);
+
+  supprs.insert(supprs.end(), wl_suppr.begin(), wl_suppr.end());
 
   add_read_context_suppressions(read_ctxt, supprs);
 }
 
+/// Configure the abigail::xml_reacher::read_context based on the
+/// relevant command-line options.
+///
+/// @param ctxt the read context to configure.
+///
+/// @param opts the command-line options to configure @p ctxt from.
+static void
+set_native_xml_reader_options(abigail::xml_reader::read_context& ctxt,
+			      const options& opts)
+{
+  consider_types_not_reachable_from_public_interfaces(ctxt,
+						      opts.show_all_types);
+}
+
 /// Set the regex patterns describing the functions to drop from the
 /// symbol table of a given corpus.
 ///
@@ -815,10 +837,6 @@ static void
 adjust_diff_context_for_kmidiff(diff_context &ctxt)
 {
   ctxt.show_linkage_names(false);
-  ctxt.show_added_fns(false);
-  ctxt.show_added_vars(false);
-  ctxt.show_added_symbols_unreferenced_by_debug_info
-    (false);
 }
 
 /// Convert options::di_root_paths{1,2} into
@@ -1059,13 +1077,14 @@ main(int argc, char* argv[])
 	  t1 = abigail::xml_reader::read_translation_unit_from_file(opts.file1,
 								    env.get());
 	  break;
-	case abigail::tools_utils::FILE_TYPE_ELF:
+	case abigail::tools_utils::FILE_TYPE_ELF: // fall through
 	case abigail::tools_utils::FILE_TYPE_AR:
 	  {
 	    abigail::dwarf_reader::read_context_sptr ctxt =
 	      abigail::dwarf_reader::create_read_context
-	      (opts.file1, opts.prepared_di_root_paths1, env.get(),
-	       /*readalltypes*/false, opts.linux_kernel_mode);
+	      (opts.file1, opts.prepared_di_root_paths1,
+	       env.get(), /*read_all_types=*/opts.show_all_types,
+	       opts.linux_kernel_mode);
 	    assert(ctxt);
 
 	    abigail::dwarf_reader::set_show_stats(*ctxt, opts.show_stats);
@@ -1087,6 +1106,7 @@ main(int argc, char* argv[])
 								  env.get());
 	    assert(ctxt);
 	    set_suppressions(*ctxt, opts);
+	    set_native_xml_reader_options(*ctxt, opts);
 	    c1 = abigail::xml_reader::read_corpus_from_input(*ctxt);
 	    if (!c1)
 	      return handle_error(c1_status, /*ctxt=*/0,
@@ -1100,6 +1120,7 @@ main(int argc, char* argv[])
 								  env.get());
 	    assert(ctxt);
 	    set_suppressions(*ctxt, opts);
+	    set_native_xml_reader_options(*ctxt, opts);
 	    g1 = abigail::xml_reader::read_corpus_group_from_input(*ctxt);
 	    if (!g1)
 	      return handle_error(c1_status, /*ctxt=*/0,
@@ -1132,13 +1153,14 @@ main(int argc, char* argv[])
 	  t2 = abigail::xml_reader::read_translation_unit_from_file(opts.file2,
 								    env.get());
 	  break;
-	case abigail::tools_utils::FILE_TYPE_ELF:
+	case abigail::tools_utils::FILE_TYPE_ELF: // Fall through
 	case abigail::tools_utils::FILE_TYPE_AR:
 	  {
 	    abigail::dwarf_reader::read_context_sptr ctxt =
 	      abigail::dwarf_reader::create_read_context
-	      (opts.file2, opts.prepared_di_root_paths2, env.get(),
-	       /*readalltypes=*/false, opts.linux_kernel_mode);
+	      (opts.file2, opts.prepared_di_root_paths2,
+	       env.get(), /*read_all_types=*/opts.show_all_types,
+	       opts.linux_kernel_mode);
 	    assert(ctxt);
 	    abigail::dwarf_reader::set_show_stats(*ctxt, opts.show_stats);
 	    abigail::dwarf_reader::set_do_log(*ctxt, opts.do_log);
@@ -1160,6 +1182,7 @@ main(int argc, char* argv[])
 								  env.get());
 	    assert(ctxt);
 	    set_suppressions(*ctxt, opts);
+	    set_native_xml_reader_options(*ctxt, opts);
 	    c2 = abigail::xml_reader::read_corpus_from_input(*ctxt);
 	    if (!c2)
 	      return handle_error(c2_status, /*ctxt=*/0, argv[0], opts);
@@ -1173,6 +1196,7 @@ main(int argc, char* argv[])
 								  env.get());
 	    assert(ctxt);
 	    set_suppressions(*ctxt, opts);
+	    set_native_xml_reader_options(*ctxt, opts);
 	    g2 = abigail::xml_reader::read_corpus_group_from_input(*ctxt);
 	    if (!g2)
 	      return handle_error(c2_status, /*ctxt=*/0, argv[0], opts);
@@ -1219,8 +1243,7 @@ main(int argc, char* argv[])
 
       if (t1)
 	{
-	  diff_context_sptr diff_ctxt(new diff_context);
-	  translation_unit_diff_sptr diff = compute_diff(t1, t2, diff_ctxt);
+	  translation_unit_diff_sptr diff = compute_diff(t1, t2, ctxt);
 	  if (diff->has_changes())
 	    diff->report(cout);
 	}
