@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -35,7 +35,7 @@
 #include <iostream>
 #include <fstream>
 #include <vector>
-#include <tr1/memory>
+#include "abg-cxx-compat.h"
 #include "abg-config.h"
 #include "abg-tools-utils.h"
 #include "abg-corpus.h"
@@ -50,12 +50,13 @@ using std::cout;
 using std::ostream;
 using std::ofstream;
 using std::vector;
-using std::tr1::shared_ptr;
+using abg_compat::shared_ptr;
 using abigail::tools_utils::emit_prefix;
 using abigail::tools_utils::temp_file;
 using abigail::tools_utils::temp_file_sptr;
 using abigail::tools_utils::check_file;
 using abigail::tools_utils::build_corpus_group_from_kernel_dist_under;
+using abigail::tools_utils::timer;
 using abigail::ir::environment_sptr;
 using abigail::ir::environment;
 using abigail::corpus;
@@ -69,6 +70,8 @@ using abigail::comparison::corpus_diff_sptr;
 using abigail::comparison::compute_diff;
 using abigail::comparison::diff_context_sptr;
 using abigail::comparison::diff_context;
+using abigail::xml_writer::create_write_context;
+using abigail::xml_writer::write_context_sptr;
 using abigail::xml_writer::write_corpus;
 using abigail::xml_reader::read_corpus_from_native_xml_file;
 using abigail::dwarf_reader::read_context;
@@ -94,6 +97,8 @@ struct options
   bool			show_base_name_alt_debug_info_path;
   bool			write_architecture;
   bool			write_corpus_path;
+  bool			write_comp_dir;
+  bool			short_locs;
   bool			load_all_types;
   bool			linux_kernel_mode;
   bool			corpus_group_for_linux;
@@ -110,6 +115,8 @@ struct options
       show_base_name_alt_debug_info_path(),
       write_architecture(true),
       write_corpus_path(true),
+      write_comp_dir(true),
+      short_locs(false),
       load_all_types(),
       linux_kernel_mode(true),
       corpus_group_for_linux(false),
@@ -148,6 +155,8 @@ display_usage(const string& prog_name, ostream& out)
     << "  --no-architecture  do not emit architecture info in the output\n"
     << "  --no-corpus-path  do not take the path to the corpora into account\n"
     << "  --no-show-locs  do not show location information\n"
+    << "  --short-locs  only print filenames rather than paths\n"
+    << "  --no-comp-dir-path  do not show compilation path information\n"
     << "  --check-alternate-debug-info <elf-path>  check alternate debug info "
     "of <elf-path>\n"
     << "  --check-alternate-debug-info-base-name <elf-path>  check alternate "
@@ -255,6 +264,10 @@ parse_command_line(int argc, char* argv[], options& opts)
 	opts.write_corpus_path = false;
       else if (!strcmp(argv[i], "--no-show-locs"))
 	opts.show_locs = false;
+      else if (!strcmp(argv[i], "--short-locs"))
+	opts.short_locs = true;
+      else if (!strcmp(argv[i], "--no-comp-dir-path"))
+	opts.write_comp_dir = false;
       else if (!strcmp(argv[i], "--check-alternate-debug-info")
 	       || !strcmp(argv[i], "--check-alternate-debug-info-base-name"))
 	{
@@ -362,12 +375,12 @@ set_suppressions(read_context& read_ctxt, options& opts)
   if (suppr)
     supprs.push_back(suppr);
 
-  using abigail::tools_utils::gen_suppr_spec_from_kernel_abi_whitelist;
-  for (vector<string>::const_iterator i =
-	 opts.kabi_whitelist_paths.begin();
-       i != opts.kabi_whitelist_paths.end();
-       ++i)
-    gen_suppr_spec_from_kernel_abi_whitelist(*i, opts.kabi_whitelist_supprs);
+  using abigail::tools_utils::gen_suppr_spec_from_kernel_abi_whitelists;
+  const suppressions_type& wl_suppr =
+      gen_suppr_spec_from_kernel_abi_whitelists(opts.kabi_whitelist_paths);
+
+  opts.kabi_whitelist_supprs.insert(opts.kabi_whitelist_supprs.end(),
+				    wl_suppr.begin(), wl_suppr.end());
 
   add_read_context_suppressions(read_ctxt, supprs);
   add_read_context_suppressions(read_ctxt, opts.kabi_whitelist_supprs);
@@ -393,12 +406,26 @@ load_corpus_and_write_abixml(char* argv[],
 			     const options& opts)
 {
   int exit_code = 0;
+  timer t;
 
   read_context& ctxt = *context;
   corpus_sptr corp;
   dwarf_reader::status s = dwarf_reader::STATUS_UNKNOWN;
+  t.start();
   corp = read_corpus_from_elf(ctxt, s);
+  t.stop();
+  if (opts.do_log)
+    emit_prefix(argv[0], cerr)
+      << "read corpus from elf file in: " << t << "\n";
+
+  t.start();
   context.reset();
+  t.stop();
+
+  if (opts.do_log)
+    emit_prefix(argv[0], cerr)
+      << "reset read context in: " << t << "\n";
+
   if (!corp)
     {
       if (s == dwarf_reader::STATUS_DEBUG_INFO_NOT_FOUND)
@@ -440,17 +467,35 @@ load_corpus_and_write_abixml(char* argv[],
     }
   else
     {
+      t.start();
+      const write_context_sptr& write_ctxt
+	  = create_write_context(corp->get_environment(), cout);
+      set_common_options(*write_ctxt, opts);
+      t.stop();
+
+      if (opts.do_log)
+	emit_prefix(argv[0], cerr)
+	  << "created & initialized write context in: "
+	  << t << "\n";
+
       if (opts.abidiff)
 	{
 	  // Save the abi in abixml format in a temporary file, read
 	  // it back, and compare the ABI of what we've read back
 	  // against the ABI of the input ELF file.
 	  temp_file_sptr tmp_file = temp_file::create();
-	  write_corpus(corp, 0, tmp_file->get_stream(), opts.annotate);
+	  set_ostream(*write_ctxt, tmp_file->get_stream());
+	  write_corpus(*write_ctxt, corp, 0);
 	  tmp_file->get_stream().flush();
+	  t.start();
 	  corpus_sptr corp2 =
 	    read_corpus_from_native_xml_file(tmp_file->get_path(),
 					     env.get());
+	  t.stop();
+	  if (opts.do_log)
+	    emit_prefix(argv[0], cerr)
+	      << "Read corpus in: " << t << "\n";
+
 	  if (!corp2)
 	    {
 	      emit_prefix(argv[0], cerr)
@@ -458,14 +503,26 @@ load_corpus_and_write_abixml(char* argv[],
 		"elf file back\n";
 	      return 1;
 	    }
+
 	  diff_context_sptr ctxt(new diff_context);
 	  set_diff_context(ctxt);
 	  ctxt->show_locs(opts.show_locs);
+	  t.start();
 	  corpus_diff_sptr diff = compute_diff(corp, corp2, ctxt);
+	  t.stop();
+	  if (opts.do_log)
+	    emit_prefix(argv[0], cerr)
+	      << "computed diff in: " << t << "\n";
+
 	  bool has_error = diff->has_incompatible_changes();
 	  if (has_error)
 	    {
+	      t.start();
 	      diff->report(cerr);
+	      t.stop();
+	      if (opts.do_log)
+		emit_prefix(argv[0], cerr)
+		  << "emitted report in: " << t << "\n";
 	      return 1;
 	    }
 	  return 0;
@@ -474,11 +531,6 @@ load_corpus_and_write_abixml(char* argv[],
       if (opts.noout)
 	return 0;
 
-      if (!opts.write_architecture)
-	corp->set_architecture_name("");
-      if (!opts.write_corpus_path)
-	corp->set_path("");
-
       if (!opts.out_file_path.empty())
 	{
 	  ofstream of(opts.out_file_path.c_str(), std::ios_base::trunc);
@@ -489,23 +541,24 @@ load_corpus_and_write_abixml(char* argv[],
 		<< opts.out_file_path << "'\n";
 	      return 1;
 	    }
-	  abigail::xml_writer::write_context_sptr write_ctxt =
-	    abigail::xml_writer::create_write_context(corp->get_environment(),
-						      of);
-	  abigail::xml_writer::set_show_locs(*write_ctxt, opts.show_locs);
-	  abigail::xml_writer::set_annotate(*write_ctxt, opts.annotate);
-	  abigail::xml_writer::write_corpus(corp, 0, *write_ctxt);
+	  set_ostream(*write_ctxt, of);
+	  t.start();
+	  write_corpus(*write_ctxt, corp, 0);
+	  t.stop();
+	  if (opts.do_log)
+	    emit_prefix(argv[0], cerr)
+	      << "emitted abixml output in: " << t << "\n";
 	  of.close();
 	  return 0;
 	}
       else
 	{
-	  abigail::xml_writer::write_context_sptr write_ctxt =
-	    abigail::xml_writer::create_write_context(corp->get_environment(),
-						      cout);
-	  abigail::xml_writer::set_show_locs(*write_ctxt, opts.show_locs);
-	  abigail::xml_writer::set_annotate(*write_ctxt, opts.annotate);
-	  exit_code = !abigail::xml_writer::write_corpus(corp, 0, *write_ctxt);
+	  t.start();
+	  exit_code = !write_corpus(*write_ctxt, corp, 0);
+	  t.stop();
+	  if (opts.do_log)
+	    emit_prefix(argv[0], cerr)
+	      << "emitted abixml out in: " << t << "\n";
 	}
     }
 
@@ -538,7 +591,15 @@ load_kernel_corpus_group_and_write_abixml(char* argv[],
     if (!abigail::tools_utils::check_file(opts.vmlinux, cerr, argv[0]))
       return 1;
 
+  timer t, global_timer;
   suppressions_type supprs;
+
+  if (opts.do_log)
+    emit_prefix(argv[0], cerr)
+      << "going to build ABI representation of the Linux Kernel ...\n";
+
+  global_timer.start();
+  t.start();
   corpus_group_sptr group =
     build_corpus_group_from_kernel_dist_under(opts.in_file_path,
 					      /*debug_info_root=*/"",
@@ -546,17 +607,24 @@ load_kernel_corpus_group_and_write_abixml(char* argv[],
 					      opts.suppression_paths,
 					      opts.kabi_whitelist_paths,
 					      supprs, opts.do_log, env);
+  t.stop();
+
+  if (opts.do_log)
+    {
+      emit_prefix(argv[0], cerr)
+	<< "built ABI representation of the Linux Kernel in: "
+	<< t << "\n";
+    }
 
   if (!group)
     return 1;
 
-  if (!opts.write_architecture)
-    group->set_architecture_name("");
-  if (!opts.write_corpus_path)
-    group->set_path("");
-
   if (!opts.noout)
     {
+      const xml_writer::write_context_sptr& ctxt
+	  = xml_writer::create_write_context(group->get_environment(), cout);
+      set_common_options(*ctxt, opts);
+
       if (!opts.out_file_path.empty())
 	{
 	  ofstream of(opts.out_file_path.c_str(), std::ios_base::trunc);
@@ -567,14 +635,36 @@ load_kernel_corpus_group_and_write_abixml(char* argv[],
 		<< opts.out_file_path << "'\n";
 	      return 1;
 	    }
-	  exit_code = !xml_writer::write_corpus_group(group, 0, of,
-						      opts.annotate);
+
+	  if (opts.do_log)
+	    emit_prefix(argv[0], cerr)
+	      << "emitting the abixml output ...\n";
+	  set_ostream(*ctxt, of);
+	  t.start();
+	  exit_code = !write_corpus_group(*ctxt, group, 0);
+	  t.stop();
+	  if (opts.do_log)
+	    emit_prefix(argv[0], cerr)
+	      << "emitted abixml output in: " << t << "\n";
 	}
       else
-	exit_code = !xml_writer::write_corpus_group(group, 0, cout,
-						    opts.annotate);
+	{
+	  if (opts.do_log)
+	    emit_prefix(argv[0], cerr)
+	      << "emitting the abixml output ...\n";
+	  t.start();
+	  exit_code = !write_corpus_group(*ctxt, group, 0);
+	  t.stop();
+	  if (opts.do_log)
+	    emit_prefix(argv[0], cerr)
+	      << "emitted abixml output in: " << t << "\n";
+	}
     }
 
+  global_timer.stop();
+  if (opts.do_log)
+    emit_prefix(argv[0], cerr)
+      << "total processing done in " << global_timer << "\n";
   return exit_code;
 }
 
